# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import proxy

from otcextensions.sdk.cts.v1 import tracker as _tracker
from otcextensions.sdk.ctsv2.v2 import trace as _trace


class Proxy(proxy.Proxy):

    skip_discovery = True

    def traces(self, tracker='system', **query):
        """List all traces

        :param tracker: The name or an tracker of
            :class:`~otcextensions.sdk.cts.v1.tracker.Tracker`
        :param limit: default limit of resources
        :returns: A generator of tracker object of
            :class:`~otcextensions.sdk.cts.v1.trace.Trace`
        """
        if isinstance(tracker, str):
            tracker = self._get_resource(_tracker.Tracker, {'name': tracker})
        return self._list(_trace.Trace, paginated=False,
                          tracker_name=tracker.name, **query)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from otcextensions.tests.functional.sdk.elb import TestElb


class TestLoadBalancerTags(TestElb):

    def setUp(self):
        super(TestLoadBalancerTags, self).setUp()

        self.create_network()
        self.create_load_balancer()

    def test_01_list_tags(self):
        query = {}
        tags = list(self.client.load_balancer_tags(
            load_balancer=TestElb.load_balancer.id,
            **query))
        self.assertGreaterEqual(len(tags), 0)

    def test_02_create_tag(self):
        kv = {
            'key': 'key1',
            'value': 'value1'
        }
        tag = self.client.create_load_balancer_tag(
            load_balancer=TestElb.load_balancer.id,
            **kv)
        self.assertIsNotNone(tag)
        self.assertEqual(kv['key'], tag.key)
        self.assertEqual(kv['value'], tag.value)

    def test_03_delete_tag(self):
        key = 'key1'
        tag = self.client.delete_load_balancer_tag(
            load_balancer=TestElb.load_balancer.id,
            key=key
        )
        self.assertIsNotNone(tag)

        self.client.delete_load_balancer(
            TestElb.load_balancer
        )
        TestElb.load_balancer = None
        self.addCleanup(self.destroy_network, TestElb.network)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from keystoneauth1 import adapter

import copy
import mock

from openstack.tests.unit import base
from otcextensions.sdk.vpc.v1 import peering


IDENTIFIER = 'ID'
EXAMPLE = {
    "name": "test",
    "id": "22b76469-08e3-4937-8c1d-7aad34892be1",
    "request_vpc_info": {
        "vpc_id": "9daeac7c-a98f-430f-8e38-67f9c044e299",
        "tenant_id": "f65e9ebc-ed5d-418b-a931-9a723718ba4e"
    },
    "accept_vpc_info": {
        "vpc_id": "f583c072-0bb8-4e19-afb2-afb7c1693be5",
        "tenant_id": "f65e9ebc-ed5d-418b-a931-9a723718ba4e"
    },
    "status": "ACTIVE"
}


class TestPeering(base.TestCase):

    def setUp(self):
        super(TestPeering, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.put = mock.Mock()

    def test_basic(self):
        sot = peering.Peering()
        self.assertEqual('peering', sot.resource_key)
        self.assertEqual('peerings', sot.resources_key)
        path = '/v2.0/vpc/peerings'

        self.assertEqual(path, sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):
        sot = peering.Peering(**EXAMPLE)
        self.assertEqual(EXAMPLE['status'], sot.status)
        self.assertEqual(EXAMPLE['request_vpc_info'], sot.local_vpc_info)
        self.assertEqual(EXAMPLE['accept_vpc_info'], sot.peer_vpc_info)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['name'], sot.name)

    def test_set_peering(self):
        sot = peering.Peering(id=IDENTIFIER)

        resp = mock.Mock()
        resp.body = {
            "name": "test",
            "id": "22b76469-08e3-4937-8c1d-7aad34892be1",
            "request_vpc_info": {
                "vpc_id": "9daeac7c-a98f-430f-8e38-67f9c044e299",
                "tenant_id": "f65e9ebc-ed5d-418b-a931-9a723718ba4e"
            },
            "accept_vpc_info": {
                "vpc_id": "f583c072-0bb8-4e19-afb2-afb7c1693be5",
                "tenant_id": "f65e9ebc-ed5d-418b-a931-9a723718ba4e"
            },
            "status": "REJECTED"
        }
        resp.json = mock.Mock(return_value=copy.deepcopy(resp.body))
        resp.headers = {}
        resp.status_code = 200
        self.sess.put.return_value = resp

        response = sot._set_peering(self.sess, 'reject')
        self.sess.put.assert_called_with(
            'v2.0/vpc/peerings/ID/reject'
        )
        self.assertEqual(resp.body['name'], response.name)
        self.assertEqual(resp.body['id'], response.id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import mock
from openstackclient.tests.unit import utils as tests_utils
from osc_lib import exceptions

from otcextensions.common import cli_utils
from otcextensions.osclient.modelartsv2.v2 import dataset_sample
from otcextensions.tests.unit.osclient.modelartsv2.v2 import fakes

_COLUMNS = (
    "annotated_by",
    "id",
    "labels",
    "metadata",
    "preview",
    "sample_id",
    "sample_status",
    "sample_time",
    "sample_type",
    "source",
)


class TestAddDatasetSamples(fakes.TestModelartsv2):
    columns = (
        "Success",
        "Results",
    )
    _dataset = fakes.FakeDataset.create_one()
    _sample = fakes.FakeDatasetSample.create_one()
    formatters = {
        "Results": cli_utils.YamlFormat,
    }

    data = fakes.gen_data(_sample, columns, formatters)

    def setUp(self):
        super(TestAddDatasetSamples, self).setUp()

        self.cmd = dataset_sample.AddDatasetSamples(self.app, None)

        self.client.find_dataset = mock.Mock(return_value=self._dataset)
        self.client.add_dataset_samples = mock.Mock(return_value=self._sample)

    def test_create(self):
        arglist = [
            "dataset-id",
            "--file-path",
            "1",
            "--directory-path",
            "2",
            "--encoding",
            "UTF-8",
            "--metadata",
            "k1=v1",
            "--metadata",
            "k2=v2",
            "--sample-type",
            "0",
            "--data-source-path",
            "3",
            "--data-source-type",
            "0",
            "--data-source-info",
            "port=1,input=2",
            "--schema-map",
            "src_name=a,dest_name=b",
            "--label",
            "name=1,type=2",
            "--data-with-column-header",
            "--to-be-confirmed",
        ]
        verifylist = [
            ("dataset", "dataset-id"),
            ("file_path", "1"),
            ("directory_path", "2"),
            ("encoding", "UTF-8"),
            ("metadata", {"k1": "v1", "k2": "v2"}),
            ("sample_type", 0),
            ("data_source_path", "3"),
            ("data_source_type", 0),
            ("data_source_info", [{"port": "1", "input": "2"}]),
            ("schema_map", [{"src_name": "a", "dest_name": "b"}]),
            ("labels", [{"name": "1", "type": "2"}]),
            ("data_with_column_header", True),
            ("to_be_confirmed", True),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)
        self.client.find_dataset.assert_called_with(
            "dataset-id", ignore_missing=False
        )
        self.client.add_dataset_samples.assert_called_with(
            self._dataset.id,
            file_path="1",
            directory_path="2",
            encoding="UTF-8",
            sample_type=0,
            metadata={"k1": "v1", "k2": "v2"},
            labels=[{"name": "1", "type": "2"}],
            data_source={
                "data_path": "3",
                "schema_map": [{"src_name": "a", "dest_name": "b"}],
                "source_info": {"port": "1", "input": "2"},
                "with_column_header": True,
            },
        )
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestListDatasetSamples(fakes.TestModelartsv2):
    dataset = fakes.FakeDataset.create_one()
    objects = fakes.FakeDatasetSample.create_multiple(3)

    column_list_headers = (
        "Id",
        "Sample Type",
        "Sample Status",
        "Sample Time",
    )

    data = []

    for s in objects:
        data.append(
            (
                s.id,
                dataset_sample.SampleType(s.sample_type),
                s.sample_status,
                cli_utils.UnixTimestampFormatter(s.sample_time),
            )
        )

    def setUp(self):
        super(TestListDatasetSamples, self).setUp()

        self.cmd = dataset_sample.ListDatasetSamples(self.app, None)

        self.client.find_dataset = mock.Mock(return_value=self.dataset)
        self.client.dataset_samples = mock.Mock()
        self.client.api_mock = self.client.dataset_samples

    def test_list(self):
        arglist = ["dataset-name"]

        verifylist = [
            ("dataset", "dataset-name"),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_dataset.assert_called_with(
            "dataset-name", ignore_missing=False
        )
        self.client.api_mock.assert_called_with(self.dataset.id)

        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(self.data, list(data))

    def test_list_args(self):
        arglist = [
            "dataset-name",
            "--email",
            "1",
            "--high-score",
            "2",
            "--label-name",
            "3",
            "--label-type",
            "1",
            "--limit",
            "5",
            "--locale",
            "6",
            "--low-score",
            "7",
            "--offset",
            "8",
            "--order",
            "asc",
            "--preview",
            "--process-parameter",
            "11",
            "--sample-state",
            "accepted",
            "--sample-type",
            "0",
            "--search-conditions",
            "13",
            "--version-id",
            "14",
        ]

        verifylist = [
            ("dataset", "dataset-name"),
            ("email", "1"),
            ("high_score", "2"),
            ("label_name", "3"),
            ("label_type", 1),
            ("limit", 5),
            ("locale", "6"),
            ("low_score", "7"),
            ("offset", 8),
            ("order", "asc"),
            ("preview", True),
            ("process_parameter", "11"),
            ("sample_state", "accepted"),
            ("sample_type", 0),
            ("search_conditions", "13"),
            ("version_id", "14"),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with(
            self.dataset.id,
            email="1",
            high_score="2",
            label_name="3",
            label_type=1,
            limit=5,
            locale="6",
            low_score="7",
            offset=8,
            order="asc",
            preview=True,
            process_parameter="11",
            sample_state="accepted",
            sample_type=0,
            search_conditions="13",
            version_id="14",
        )


class TestShowDatasetSample(fakes.TestModelartsv2):
    columns = _COLUMNS
    dataset = fakes.FakeDataset.create_one()
    _data = fakes.FakeDatasetSample.create_one()

    data = fakes.gen_data(_data, columns, dataset_sample._formatters)

    def setUp(self):
        super(TestShowDatasetSample, self).setUp()

        self.cmd = dataset_sample.ShowDatasetSample(self.app, None)

        self.client.find_dataset = mock.Mock(return_value=self.dataset)
        self.client.get_dataset_sample = mock.Mock(return_value=self._data)

    def test_show_no_options(self):
        arglist = []
        verifylist = []

        # Testing that a call without the required argument will fail and
        # throw a "ParserExecption"
        self.assertRaises(
            tests_utils.ParserException,
            self.check_parser,
            self.cmd,
            arglist,
            verifylist,
        )

    def test_show(self):
        arglist = [
            "dataset-name",
            "sample-id",
        ]

        verifylist = [
            ("dataset", "dataset-name"),
            ("sampleId", "sample-id"),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)
        self.client.find_dataset.assert_called_with(
            "dataset-name", ignore_missing=False
        )
        self.client.get_dataset_sample.assert_called_with(
            self.dataset.id, "sample-id"
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)

    def test_show_non_existent(self):
        arglist = [
            "dataset-name",
            "nonexisting-sample-id",
        ]

        verifylist = [
            ("dataset", "dataset-name"),
            ("sampleId", "nonexisting-sample-id"),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        get_mock_result = exceptions.CommandError("Resource Not Found")
        self.client.get_dataset_sample = mock.Mock(side_effect=get_mock_result)

        # Trigger the action
        try:
            self.cmd.take_action(parsed_args)
        except Exception as e:
            self.assertEqual("Resource Not Found", str(e))
        self.client.get_dataset_sample.assert_called_with(
            "dataset-id",
            "nonexisting-sample-id",
        )


class TestDeleteDatasetSample(fakes.TestModelartsv2):
    columns = ("results", "success")
    dataset = fakes.FakeDataset.create_one()
    _data = fakes.FakeDatasetSampleResp.create_one()
    formatters = {
        "results": cli_utils.YamlFormat,
    }
    data = fakes.gen_data(_data, columns, formatters)

    def setUp(self):
        super(TestDeleteDatasetSample, self).setUp()

        self.client.find_dataset = mock.Mock(return_value=self.dataset)
        self.client.delete_dataset_samples = mock.Mock(return_value=self._data)

        # Get the command object to test
        self.cmd = dataset_sample.DeleteDatasetSamples(self.app, None)

    def test_delete(self):
        arglist = [
            "dataset-name",
            "sample1-id",
            "sample2-id",
            "--delete-source",
        ]

        verifylist = [
            ("dataset", "dataset-name"),
            ("sampleId", ["sample1-id", "sample2-id"]),
            ("delete_source", True),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)
        self.client.find_dataset.assert_called_with(
            "dataset-name", ignore_missing=False
        )
        self.client.delete_dataset_samples.assert_called_with(
            self.dataset.id, ["sample1-id", "sample2-id"], True
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class Statistic(resource.Resource):

    resources_key = 'statistics'

    base_path = '/instances/statistic'

    # capabilities
    allow_list = True
    allow_fetch = True

    # Properties
    #: Number of times that the GET command is run.
    #: *Type: int*
    cmd_get_count = resource.Body('cmd_get_count', type=int)
    #: Number of times that the SET command is run.
    #: *Type: int*
    cmd_set_count = resource.Body('cmd_set_count', type=int)
    #: Incoming traffic of the DCS instance.
    #: Unit: kbit/s.
    #: *Type: str*
    input_kbps = resource.Body('input_kbps')
    #: Instance Id
    instance_id = resource.Body('instance_id', alternate_id=True)
    #: Number of data items stored in the cache.
    #: *Type: int*
    keys = resource.Body('keys', type=int)
    #: Overall memory size.
    #: Unit: MB.
    #: *Type: int*
    max_memory = resource.Body('max_memory', type=int)
    #: Outgoing traffic of the DCS instance.
    #: Unit: kbit/s.
    #: *Type: str*
    output_kbps = resource.Body('output_kbps')
    #: CPU Usage %
    #: *Type: str*
    used_cpu = resource.Body('used_cpu')
    #: Size of the used memory.
    #: Unit: MB.
    #: *Type: int*
    used_memory = resource.Body('used_memory', type=int)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import mock
from unittest.mock import call

from osc_lib import exceptions

from otcextensions.osclient.smn.v2 import topic
from otcextensions.tests.unit.osclient.smn.v2 import fakes

from openstackclient.tests.unit import utils as tests_utils


class TestListTopic(fakes.TestSmn):

    objects = fakes.FakeTopic.create_multiple(3)

    column_list_headers = ('ID', 'Name', 'Display Name', 'Push Policy')

    columns = ('id', 'name', 'display_name', 'push_policy')

    data = []

    for s in objects:
        data.append(
            (s.id, s.name, s.display_name, s.push_policy))

    def setUp(self):
        super(TestListTopic, self).setUp()

        self.cmd = topic.ListTopic(self.app, None)

        self.client.topics = mock.Mock()
        self.client.api_mock = self.client.topics

    def test_list(self):
        arglist = []

        verifylist = []

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with()

        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(self.data, list(data))

    def test_list_args(self):
        arglist = [
            '--limit', '1',
            '--offset', '2'
        ]

        verifylist = [
            ('limit', 1),
            ('offset', 2),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with(
            limit=1,
            offset=2,
        )


class TestCreateTopic(fakes.TestSmn):

    _data = fakes.FakeTopic.create_one()

    columns = (
        'request_id',
        'topic_urn'
    )

    data = fakes.gen_data(_data, columns)

    def setUp(self):
        super(TestCreateTopic, self).setUp()

        self.cmd = topic.CreateTopic(self.app, None)

        self.client.create_topic = mock.Mock(return_value=self._data)

    def test_create(self):
        arglist = [
            'test-topic',
            '--display-name', 'topic_display_name',
        ]
        verifylist = [
            ('name', 'test-topic'),
            ('display_name', 'topic_display_name'),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.create_topic.assert_called_with(
            name='test-topic',
            display_name='topic_display_name',
        )
        self.assertEqual(self.columns, columns)


class TestUpdateTopic(fakes.TestSmn):

    _data = fakes.FakeTopic.create_one()

    columns = (
        'create_time',
        'display_name',
        'id',
        'name',
        'push_policy',
        'request_id',
        'update_time')

    data = fakes.gen_data(_data, columns)

    def setUp(self):
        super(TestUpdateTopic, self).setUp()

        self.cmd = topic.UpdateTopic(self.app, None)

        self.client.find_topic = mock.Mock(return_value=self._data)
        self.client.update_topic = mock.Mock(return_value=self._data)

    def test_update(self):
        arglist = [
            self._data.name,
            '--display-name', 'topic display name updated',
        ]
        verifylist = [
            ('topic', self._data.name),
            ('display_name', 'topic display name updated'),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_topic.assert_called_with(self._data.name)
        self.client.update_topic.assert_called_with(
            self._data,
            display_name='topic display name updated',
        )
        self.assertEqual(self.columns, columns)


class TestShowTopic(fakes.TestSmn):

    _data = fakes.FakeTopic.create_one()

    columns = (
        'create_time',
        'display_name',
        'id',
        'name',
        'push_policy',
        'request_id',
        'update_time')

    data = fakes.gen_data(_data, columns)

    def setUp(self):
        super(TestShowTopic, self).setUp()

        self.cmd = topic.ShowTopic(self.app, None)

        self.client.find_topic = mock.Mock(return_value=self._data)

    def test_show_no_options(self):
        arglist = []
        verifylist = []

        # Testing that a call without the required argument will fail and
        # throw a "ParserExecption"
        self.assertRaises(tests_utils.ParserException,
                          self.check_parser, self.cmd, arglist, verifylist)

    def test_show(self):
        arglist = [
            self._data.id,
        ]

        verifylist = [
            ('topic', self._data.id),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)
        self.client.find_topic.assert_called_with(self._data.id)

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)

    def test_show_non_existent(self):
        arglist = [
            'unexist_topic',
        ]

        verifylist = [
            ('topic', 'unexist_topic'),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        find_mock_result = exceptions.CommandError('Resource Not Found')
        self.client.find_topic = (
            mock.Mock(side_effect=find_mock_result)
        )

        # Trigger the action
        try:
            self.cmd.take_action(parsed_args)
        except Exception as e:
            self.assertEqual('Resource Not Found', str(e))
        self.client.find_topic.assert_called_with('unexist_topic')


class TestDeleteTopic(fakes.TestSmn):

    _data = fakes.FakeTopic.create_multiple(2)

    def setUp(self):
        super(TestDeleteTopic, self).setUp()

        self.client.delete_topic = mock.Mock(return_value=None)

        # Get the command object to test
        self.cmd = topic.DeleteTopic(self.app, None)

    def test_delete(self):
        arglist = [
            self._data[0].name,
        ]

        verifylist = [
            ('topic', [self._data[0].name]),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.client.find_topic = (
            mock.Mock(return_value=self._data[0])
        )

        # Trigger the action
        result = self.cmd.take_action(parsed_args)
        self.client.delete_topic.assert_called_with(self._data[0])
        self.assertIsNone(result)

    def test_multiple_delete(self):
        arglist = []

        for data in self._data:
            arglist.append(data.name)

        verifylist = [
            ('topic', arglist),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        find_mock_result = self._data
        self.client.find_topic = (
            mock.Mock(side_effect=find_mock_result)
        )

        # Trigger the action
        result = self.cmd.take_action(parsed_args)

        calls = []
        for data in self._data:
            calls.append(call(data))
        self.client.delete_topic.assert_has_calls(calls)
        self.assertIsNone(result)

    def test_multiple_delete_with_exception(self):
        arglist = [
            self._data[0].name,
            'unexist_topic',
        ]
        verifylist = [
            ('topic', arglist),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        find_mock_result = [self._data[0], exceptions.CommandError]
        self.client.find_topic = (
            mock.Mock(side_effect=find_mock_result)
        )

        # Trigger the action
        try:
            self.cmd.take_action(parsed_args)
        except Exception as e:
            self.assertEqual('1 of 2 SMN Topic(s) failed to delete.', str(e))

        self.client.find_topic.assert_any_call(self._data[0].name)
        self.client.find_topic.assert_any_call('unexist_topic')
        self.client.delete_topic.assert_called_once_with(self._data[0])



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
"""Create VPC Endpoint Service."""
import openstack

openstack.enable_logging(True)
conn = openstack.connect(cloud='otc')

attrs = {
    'port_id': 'port-id',
    'vpc_id': 'router-id',
    'service_name': 'test-service',
    'approval_enabled': False,
    'service_type': 'interface',
    'server_type': 'VM',
    'ports': [{'client_port': 8080, 'server_port': 90, 'protocol': 'TCP'}],
}

endpoint_service = conn.vpcep.create_service(**attrs)
print(endpoint_service)



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
"""Create a dataset label from attributes."""
import openstack

openstack.enable_logging(True)
conn = openstack.connect(cloud="otc")
dataset_id = "heZw7Oh7Ha0eiFIzkm8"
attrs = {
    "labels": [
        {
            "name": "Cat",
            "property": {
                "@modelarts:color": "#3399ff",
                "@modelarts:default_shape": "bndbox",
            },
        }
    ]
}

dataset_label = conn.modelartsv2.create_dataset_label(dataset_id, **attrs)
print(dataset_label)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class Message(resource.Resource):
    base_path = '/notifications/topics/%(topic_urn)s/publish'

    # capabilities
    allow_create = True

    #: Message ID, which is unique
    message_id = resource.Body('message_id')
    #: Request ID, which is unique.
    request_id = resource.Body('request_id')
    #: Message subject, which is used as the email
    #: subject when you publish email messages.
    subject = resource.Body('subject')
    #: Message content.
    #: The message content is a UTF-8-coded character string of
    #: no more than 256 KB. For SMS subscribers, if the content
    #: exceeds 256 bytes, the system will divide it into multiple
    #: messages and send only the first two.
    message = resource.Body('message')
    #: Message structure, which contains JSON character strings.
    #: Specify protocols in the structure, which can be http,
    #: https, email, dms, and sms.
    message_structure = resource.Body('message_structure')
    #: Message template name, which can be obtained according
    #: to Querying Message Templates
    message_template_name = resource.Body('message_template_name')
    #: Time-to-live (TTL) of a message, specifically, the maximum time
    #: period for retaining the message in the system
    time_to_live = resource.Body('time_to_live')
    #: Resource identifier of a topic, which is unique
    topic_urn = resource.URI('topic_urn')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from openstack import resource
from openstack import utils


class ScheduleSpec(resource.Resource):
    #: Auto stop duration.
    duration = resource.Body("duration", type=int)
    #: Unit of auto stop duration.
    time_unit = resource.Body("time_unit")
    #: Set this parameter to stop.
    type = resource.Body("type")


class FlavorSpec(resource.Resource):
    #: Resource specification code of a visualization job.
    code = resource.Body("code")


class VisualizationJob(resource.Resource):
    base_path = "/visualization-jobs"

    resources_key = "jobs"

    # capabilities
    allow_create = True
    allow_list = True
    allow_commit = True
    allow_delete = True
    allow_fetch = True

    _query_mapping = resource.QueryParameters(
        "order",
        "offset",
        "limit",
        "search_content",
        "sort_by",
        "status",
        offset="page",
        limit="per_page",
    )

    # Parameters
    #: Time when a visualization job is created, in timestamp format.
    created_at = resource.Body("create_time", type=int)
    #: Visualization job running duration, in milliseconds.
    duration = resource.Body("duration", type=int)
    #: Error code of a failed API call.
    error_code = resource.Body("error_code")
    #: Error message of a failed API call.
    error_message = resource.Body("error_message")
    #: Specifications when a visualization job is created.
    flavor = resource.Body("flavor", type=FlavorSpec)
    #: Whether the request is successful
    is_success = resource.Body("is_success", type=bool)
    #: Description of a visualization job.
    job_desc = resource.Body("job_desc")
    #: ID of a visualization job.
    job_id = resource.Body("job_id", type=int, alternate_id=True)
    #: Name of a visualization job.
    job_name = resource.Body("job_name")
    #: Type of a visualization job.
    job_type = resource.Body("job_type")
    #: Name of a visualization job.
    name = resource.Body("name", alias="job_name")
    #: Remaining auto stop duration
    remaining_duration = resource.Body("remaining_duration", type=float)
    #: Charged resource ID of a visualization job.
    resource_id = resource.Body("resource_id")
    #: Auto stop setting.
    schedule = resource.Body("schedule", type=ScheduleSpec)
    #: Endpoint of a visualization job.
    service_url = resource.Body("service_url")
    #: Status of a visualization job.
    status = resource.Body("status", type=int)
    #: OBS path of the visualization job output file.
    train_url = resource.Body("train_url")

    def _action(self, session, action, body=None):
        """Preform actions given the message body."""
        uri = utils.urljoin("visualization-jobs", self.id, action)
        response = session.post(uri, json=body)
        self._translate_response(response)
        return self

    def restart(self, session):
        """Restart the Visualization job."""
        self._action(session, "restart")

    def stop(self, session):
        """Restart the Visualization job."""
        self._action(session, "stop")



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
"""DWS Snapshot action implementations"""

import logging

from osc_lib import exceptions
from osc_lib import utils
from osc_lib.command import command

from otcextensions.common import sdk_utils
from otcextensions.i18n import _
from otcextensions.osclient.dws.v1 import cluster as _cluster

LOG = logging.getLogger(__name__)


def _get_columns(item):
    column_map = {}
    hidden = [
        'location',
    ]
    return sdk_utils.get_osc_show_columns_for_sdk_resource(
        item, column_map, hidden
    )


def translate_response(func):
    def new(self, *args, **kwargs):
        obj = func(self, *args, **kwargs)
        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return (display_columns, data)

    new.__name__ = func.__name__
    new.__doc__ = func.__doc__
    return new


class ListSnapshots(command.Lister):
    _description = _('List DWS Backups.')

    columns = (
        'ID',
        'Name',
        'Type',
        'Cluster Id',
    )

    def get_parser(self, prog_name):
        parser = super(ListSnapshots, self).get_parser(prog_name)
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dws
        data = client.snapshots()

        return (
            self.columns,
            (utils.get_item_properties(s, self.columns) for s in data),
        )


class CreateSnapshot(command.ShowOne):
    _description = _('Create snapshot for a specified cluster.')

    def get_parser(self, prog_name):
        parser = super(CreateSnapshot, self).get_parser(prog_name)

        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_(
                'ID or name of the cluster for which you want to '
                'create a snapshot.'
            ),
        )
        parser.add_argument(
            'name',
            metavar='<name>',
            help=_(
                'Snapshot name, which must be unique and start with a '
                'letter. It consists of 4 to 64 characters, which are '
                'case-insensitive and contain letters, digits, '
                'hyphens (-), and underscores (_) only.'
            ),
        )

        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_(
                'Snapshot description. If no value is specified, '
                'the description is empty. Enter a maximum of 256 '
                'characters. The following special characters are '
                'not allowed: !<>\'=&"'
            ),
        )
        parser.add_argument(
            '--wait',
            action='store_true',
            help=('Wait for the Cluster snapshotting task to finish.'),
        )
        parser.add_argument(
            '--timeout',
            metavar='<timeout>',
            type=int,
            default=900,
            help=_('Timeout for the wait in seconds (Default 900 seconds).'),
        )
        return parser

    @translate_response
    def take_action(self, parsed_args):
        client = self.app.client_manager.dws
        cluster = client.find_cluster(
            parsed_args.cluster, ignore_missing=False
        )

        attrs = {'name': parsed_args.name, 'cluster_id': cluster.id}
        if parsed_args.description:
            attrs['description'] = parsed_args.description

        obj = client.create_snapshot(**attrs)
        if parsed_args.wait:
            client.wait_for_cluster(cluster.id, wait=parsed_args.timeout)

        return client.get_snapshot(obj.id)


class ShowSnapshot(command.ShowOne):
    _description = _('Show details of a DWS snapshot.')

    def get_parser(self, prog_name):
        parser = super(ShowSnapshot, self).get_parser(prog_name)
        parser.add_argument(
            'snapshot', metavar='<snapshot>', help=_('Snapshot name or ID.')
        )
        return parser

    @translate_response
    def take_action(self, parsed_args):
        client = self.app.client_manager.dws

        return client.find_snapshot(parsed_args.snapshot, ignore_missing=False)


class RestoreSnapshot(command.ShowOne):
    _description = _('Restore clusters using the snapshot.')

    def get_parser(self, prog_name):
        parser = super(RestoreSnapshot, self).get_parser(prog_name)
        parser.add_argument('name', metavar='<name>', help=_('Cluster Name.'))
        parser.add_argument(
            '--snapshot-id',
            metavar='<snapshot_id>',
            required=True,
            help=_('ID of the snapshot to be restored.'),
        )
        parser.add_argument(
            '--availability-zone',
            metavar='<availability_zone>',
            help=_(
                'AZ of a cluster. The default value is the same '
                'as that of the original cluster.'
            ),
        )
        network_group = parser.add_argument_group('Network Parameters')
        network_group.add_argument(
            '--router-id',
            metavar='<router_id>',
            dest='vpc_id',
            help=_(
                'Router ID, which is used for configuring cluster '
                'network. The default value is the same as that of '
                'the original cluster.'
            ),
        )
        network_group.add_argument(
            '--network-id',
            metavar='<network_id>',
            dest='subnet_id',
            help=_(
                'Network ID, which is used for configuring cluster '
                'network. The default value is the same as that of '
                'the original cluster.'
            ),
        )
        network_group.add_argument(
            '--security-group-id',
            metavar='<security_group_id>',
            help=_(
                'Security group ID, which is used for configuring '
                'cluster network. The default value is the same as '
                'that of the original cluster.'
            ),
        )
        parser.add_argument(
            '--port',
            metavar='<port>',
            type=int,
            help=_(
                'Service port of a cluster. The value ranges from '
                '8000 to 30000. The default value is 8000.'
            ),
        )
        parser.add_argument(
            '--floating-ip',
            metavar='<floating_ip>',
            help=_(
                'Public IP address. If the parameter is not specified, '
                'public connection is not used by default.\n'
                'Possible values can be:\n'
                '- "auto" - To automatically assign Floating IP.\n'
                '- "ID" or "IP" of existing floating ip.'
            ),
        )
        parser.add_argument(
            '--enterprise-project-id',
            metavar='<enterprise_project_id>',
            help=_(
                'Enterprise project. The default '
                'enterprise project ID is 0.'
            ),
        )
        parser.add_argument(
            '--wait',
            action='store_true',
            help=('Wait for the status of Restored cluster to be available.'),
        )
        parser.add_argument(
            '--timeout',
            metavar='<timeout>',
            type=int,
            default=1800,
            help=_('Timeout for the wait in seconds (Default 1800 seconds).'),
        )
        return parser

    @_cluster.translate_response
    def take_action(self, parsed_args):
        client = self.app.client_manager.dws

        attrs = {}
        for arg in (
            'name',
            'vpc_id',
            'subnet_id',
            'security_group_id',
            'port',
            'availability_zone',
            'enterprise_project_id',
        ):
            val = getattr(parsed_args, arg)
            if val:
                attrs[arg] = val

        floating_ip = parsed_args.floating_ip

        if floating_ip and floating_ip.lower() == 'auto':
            attrs['public_ip'] = {
                'public_bind_type': 'auto_assign',
                'eip_id': ''
            }
        elif floating_ip:
            network_client = self.app.client_manager.network
            floating_ip_resp = network_client.find_ip(
                floating_ip, ignore_missing=False
            )
            attrs['public_ip'] = {
                'public_bind_type': 'bind_existing',
                'eip_id': floating_ip_resp.id,
            }

        obj = client.restore_snapshot(parsed_args.snapshot_id, **attrs)

        if parsed_args.wait:
            client.wait_for_cluster(obj.id, parsed_args.timeout)
        return client.get_cluster(obj.id)


class DeleteSnapshot(command.Command):
    _description = _('Delete specified manual snapshot(s).')

    def get_parser(self, prog_name):
        parser = super(DeleteSnapshot, self).get_parser(prog_name)
        parser.add_argument(
            'snapshot',
            metavar='<snapshot>',
            nargs='+',
            help=_('ID or Name of the Snapshot(s) to be deleted.'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dws
        result = 0
        for name_or_id in parsed_args.snapshot:
            try:
                snapshot = client.find_snapshot(
                    name_or_id, ignore_missing=False
                )
                client.delete_snapshot(snapshot.id)
            except Exception as e:
                result += 1
                LOG.error(
                    _(
                        'Failed to delete snapshot(s) with '
                        "ID or Name '%(snapshot)s': %(e)s"
                    ),
                    {'snapshot': name_or_id, 'e': e},
                )
        if result > 0:
            total = len(parsed_args.snapshot)
            msg = _(
                '%(result)s of %(total)s Snapshot(s) failed ' 'to delete.'
            ) % {'result': result, 'total': total}
            raise exceptions.CommandError(msg)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from dataclasses import dataclass
from typing import List

from openstack import exceptions


@dataclass
class FlavorSpec:
    """
    Flavor specification

    * type: node type.
    * num: node quantity.
    * storage: [Optional] disk type.
    * size: [Optional] disk size.
    * spec_code: specification code.
    """
    type: str
    num: str
    spec_code: str
    storage: str = None
    size: int = None


class DdsMixin:
    def create_dds_instance(
            self,
            name: str,
            router,
            network,
            security_group,
            flavors: List[FlavorSpec],
            password: str,
            region='eu-de',
            availability_zone='eu-de-01',
            datastore_type='DDS-Community',
            datastore_storage_engine='wiredTiger',
            datastore_version='3.2',
            mode='ReplicaSet',
            disk_encryption_id: str = None,
            backup_timeframe: str = None,
            backup_keepdays: str = None,
            ssl_option: str = None
    ):
        """
        Create DDS instance

        :param name: instance name, dict(required=True, type=str)
        :param router: router name or id, dict(type=str)
        :param network: network name or id, dict(type=str)
        :param security_group: sg name or id, dict(type=str)
        :param flavors: list of flavors,
            dict(type=list, elements=dict)
        :param password: password, dict(type=str, no_log=True)
        :param region: dict(type=str, default='eu-de')
        :param availability_zone: dict(type=str, default='eu-de-01'),
        :param datastore_type: dict(type=str, default='DDS-Community')
        :param datastore_storage_engine:
            dict(type=str, default='wiredTiger')
        :param datastore_version: dict(type=str, choices=['3.2', '3.4']),
        :param mode: dict(choices=['Sharding', 'ReplicaSet'],
            default='ReplicaSet')
        :param disk_encryption_id: dict(type=str)
        :param backup_timeframe: dict(type=str)
        :param backup_keepdays: dict(type=str)
        :param ssl_option: dict(type=str, choices=['0', '1'])

        :returns: The results of instance creation
        :rtype: :class:`~otcextensions.sdk.dds.v3.instance.Instance`
        """

        attrs = {}
        attrs['name'] = name
        attrs['region'] = region
        attrs['availability_zone'] = availability_zone
        attrs['password'] = password

        datastore = {
            'type': datastore_type,
            'version': datastore_version,
            'storage_engine': datastore_storage_engine
        }
        attrs['datastore'] = datastore

        router_obj = self.network.find_router(router, ignore_missing=False)
        attrs['vpc_id'] = router_obj.id

        network_obj = self.network.find_network(network, ignore_missing=False)
        attrs['subnet_id'] = network_obj.id

        security_group_obj = self.network.find_security_group(
            security_group, ignore_missing=False)
        attrs['security_group_id'] = security_group_obj.id

        if disk_encryption_id:
            attrs['disk_encryption_id'] = disk_encryption_id

        if mode not in ['Sharding', 'ReplicaSet']:
            raise exceptions.SDKException(
                '`Sharding` or `ReplicaSet` are supported values'
            )
        attrs['mode'] = mode

        flavors_ref = list(self.dds.flavors(
            region=region,
            engine_name=datastore_type)
        )
        flavors_specs = [flavor.spec_code for flavor in flavors_ref]

        for flavor in flavors:
            if flavor['type'] in ['mongos', 'shard'] \
                    and flavor['num'] not in range(2, 16):
                raise exceptions.SDKException(
                    '`num` value must be in ranges from 2 to 16 '
                    'for mongos and shard'
                )
            if flavor['type'] in ['config', 'replica'] \
                    and flavor['num'] != 1:
                raise exceptions.SDKException(
                    '`num` value must be 1 '
                    'for config and replica'
                )
            if flavor['type'] == 'mongos':
                if all(k in flavor for k in ('storage', 'size')):
                    raise exceptions.SDKException(
                        '`storage` and `size` parameters'
                        ' is invalid for the mongos nodes'
                    )
            if 'size' in flavor:
                if flavor['type'] == 'replica' \
                        and not (10 <= flavor['size'] <= 2000):
                    raise exceptions.SDKException(
                        '`size` value for `replica` must be'
                        ' between 10 and 2000 GB.'
                    )
                elif flavor['type'] == 'config' \
                        and flavor['size'] != 20:
                    raise exceptions.SDKException(
                        '`size` value for `config` must be 20 GB.'
                    )
                elif not (10 <= flavor['size'] <= 1000):
                    raise exceptions.SDKException(
                        '`size` value for `shard` must be'
                        ' between 10 and 1000 GB.'
                    )
            if flavor['spec_code'] not in flavors_specs:
                raise exceptions.SDKException(
                    '`spec_code` not valid'
                )
        attrs['flavor'] = flavors

        if backup_keepdays and backup_timeframe:
            attrs['backup_strategy'] = {
                'keep_days': backup_keepdays,
                'start_time': backup_timeframe
            }
        elif backup_keepdays or backup_timeframe:
            raise exceptions.SDKException(
                '`backup_keepdays` and `backup_timeframe` must be passed'
                'together'
            )

        if ssl_option:
            attrs['ssl_option'] = ssl_option

        obj = self.dds.create_instance(**attrs)
        obj = self.dds.get_instance(obj.id)
        return obj

    def delete_dds_instance(self, instance):
        """Delete DDS instance

        :param str instance: Name or ID of instance
        """

        inst = self.dds.find_instance(
            name_or_id=instance,
            ignore_missing=False
        )
        self.dds.delete_instance(inst.id)

        return None



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid

from otcextensions.tests.functional import base


class TestFg(base.BaseFunctionalTest):
    uuid_v4 = uuid.uuid4().hex[:8]
    function_attrs = {
        'func_name': 'test-function-' + uuid_v4,
        'package': 'default',
        'runtime': 'Python3.9',
        'handler': 'index.handler',
        'timeout': 30,
        'memory_size': 128,
        'code_type': 'inline',
        'func_code': {
            'file': 'CmltcG9ydCBqc29uCgpkZWYgaGFuZGxlcihldmVudCwgY29udGV4d'
                    'Ck6CiAgICB0cnk6CiAgICAgICAgIyBQYXJzZSB0aGUgaW5wdXQgZX'
                    'ZlbnQgKGRhdGEpCiAgICAgICAgaW5wdXRfZGF0YSA9IGpzb24ubG9'
                    'hZHMoZXZlbnQpCiAgICAgICAgCiAgICAgICAgIyBQcmludCB0aGUg'
                    'cHJvdmlkZWQgZGF0YQogICAgICAgIHByaW50KCJSZWNlaXZlZCBkY'
                    'XRhOiIsIGlucHV0X2RhdGEpCiAgICAgICAgCiAgICAgICAgIyBSZX'
                    'R1cm4gYSByZXNwb25zZQogICAgICAgIHJlc3BvbnNlID0gewogICA'
                    'gICAgICAgICAic3RhdHVzQ29kZSI6IDIwMCwKICAgICAgICAgICAg'
                    'ImJvZHkiOiBmIkRhdGEgcmVjZWl2ZWQ6IHtpbnB1dF9kYXRhfSIKI'
                    'CAgICAgICB9CiAgICAgICAgcmV0dXJuIGpzb24uZHVtcHMocmVzcG'
                    '9uc2UpCgogICAgZXhjZXB0IEV4Y2VwdGlvbiBhcyBlOgogICAgICA'
                    'gICMgSGFuZGxlIGVycm9ycwogICAgICAgIGVycm9yX3Jlc3BvbnNl'
                    'ID0gewogICAgICAgICAgICAic3RhdHVzQ29kZSI6IDUwMCwKICAgI'
                    'CAgICAgICAgImJvZHkiOiBmIkVycm9yOiB7c3RyKGUpfSIKICAgIC'
                    'AgICB9CiAgICAgICAgcmV0dXJuIGpzb24uZHVtcHMoZXJyb3JfcmV'
                    'zcG9uc2UpCg=='
        }
    }

    def setUp(self):
        super(TestFg, self).setUp()
        self.client = self.conn.functiongraph



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import base
from otcextensions.sdk.apig.v2 import throttling_policy_binding

EXAMPLE_BIND = {
    "throttle_id": "0b0e8f456b8742218af75f945307173c",
    "publish_ids": ["40e7162dc6b94bbbbb1a60d2a24b1b0c"]
}


class TestThrottlingPolicyBind(base.TestCase):

    def test_basic(self):
        sot = throttling_policy_binding.ThrottlingPolicyBind()
        self.assertEqual(
            '/apigw/instances/%(gateway_id)s/throttle-bindings',
            sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_delete)
        self.assertEqual('apis', sot.resources_key)

    def test_make_it(self):
        sot = throttling_policy_binding.ThrottlingPolicyBind(**EXAMPLE_BIND)
        self.assertEqual(EXAMPLE_BIND['throttle_id'], sot.throttle_id)
        self.assertEqual(EXAMPLE_BIND['publish_ids'], sot.publish_ids)


EXAMPLE_NOTBOUND = {
    "run_env_name": "RELEASE",
    "group_name": "api_group_001",
    "publish_id": "40e7162dc6b94bbbbb1a60d2a24b1b0c",
    "group_id": "c77f5e81d9cb4424bf704ef2b0ac7600",
    "throttle_apply_id": "3e06ac135e18477e918060d3c59d6f6a",
    "name": "Api_http",
    "apply_time": "2020-08-03T12:25:52Z",
    "remark": "Web backend API",
    "run_env_id": "DEFAULT_ENVIRONMENT_RELEASE_ID",
    "id": "5f918d104dc84480a75166ba99efff21",
    "type": 1,
    "throttle_name": "throttle_demo",
    "auth_type": "APP",
    "req_uri": "/test/http"
}


class TestThrottlingPolicyNotBoundApi(base.TestCase):

    def test_basic(self):
        sot = throttling_policy_binding.NotBoundApi()
        self.assertEqual(
            '/apigw/instances/%(gateway_id)s/'
            'throttle-bindings/unbinded-apis',
            sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertEqual('apis', sot.resources_key)

    def test_make_it(self):
        sot = throttling_policy_binding.NotBoundApi(**EXAMPLE_NOTBOUND)
        self.assertEqual(
            EXAMPLE_NOTBOUND['throttle_apply_id'], sot.throttle_apply_id
        )


EXAMPLE_BOUND = {
    "id": "3437448ad06f4e0c91a224183116e965",
    "name": "throttle_demo",
    "api_call_limits": 800,
    "user_call_limits": 500,
    "app_call_limits": 300,
    "ip_call_limits": 600,
    "time_interval": 1,
    "time_unit": "SECOND",
    "create_time": "2020-07-31T08:44:02Z",
    "is_inclu_special_throttle": 2,
    "env_name": "RELEASE",
    "type": 1,
    "bind_id": "3e06ac135e18477e918060d3c59d6f6a",
    "bind_time": "2020-08-03T12:25:52Z",
    "bind_num": 0,
    "enable_adaptive_control": "FALSE"
}


class TestThrottlingPolicyBoundThrottles(base.TestCase):

    def test_basic(self):
        sot = throttling_policy_binding.BoundThrottles()
        self.assertEqual(
            '/apigw/instances/%(gateway_id)s/'
            'throttle-bindings/binded-throttles',
            sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertEqual('throttles', sot.resources_key)

    def test_make_it(self):
        sot = throttling_policy_binding.BoundThrottles(**EXAMPLE_BOUND)
        self.assertEqual(EXAMPLE_BOUND['id'], sot.id)
        self.assertEqual(EXAMPLE_BOUND['name'], sot.name)
        self.assertEqual(EXAMPLE_BOUND['api_call_limits'], sot.api_call_limits)
        self.assertEqual(
            EXAMPLE_BOUND['user_call_limits'], sot.user_call_limits
        )
        self.assertEqual(EXAMPLE_BOUND['ip_call_limits'], sot.ip_call_limits)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base
from otcextensions.sdk.apig.v2 import gateway_features

EXAMPLE = {
    "config": "on",
    "enable": True,
    "id": "db9a9260cd3e4a16a9b5747a65d3ffaa",
    "gateway_id": "eddc4d25480b4cd6b512f270a1b8b341",
    "name": "app_api_key",
    "updated_at": "2020-08-24T01:17:31.041984021Z"
}


class TestGatewayFeatures(base.TestCase):

    def test_basic(self):
        sot = gateway_features.GatewayFeatures()
        self.assertEqual(
            '/apigw/instances/%(gateway_id)s/features',
            sot.base_path
        )
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)

    def test_make_it(self):
        sot = gateway_features.GatewayFeatures(**EXAMPLE)
        self.assertEqual(EXAMPLE['config'], sot.config)
        self.assertEqual(EXAMPLE['enable'], sot.enable)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['gateway_id'], sot.gateway_id)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['updated_at'], sot.updated_at)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''DMS Instance v1 action implementations'''
from osc_lib import exceptions
from osc_lib import utils
from osc_lib.command import command

from otcextensions.common import sdk_utils
from otcextensions.i18n import _


INSTANCE_STATUS_CHOICES = ['CREATING', 'CREATEFAILED', 'RUNNING', 'ERROR',
                           'STARTING', 'RESTARTING', 'CLOSING', 'FROZEN']
RETENTION_POLICY_CHOICES = ['produce_reject', 'time_base']
STORAGE_SPEC_CHOICES = ['dms.physical.storage.high',
                        'dms.physical.storage.ultra']


def _get_columns(item):
    column_map = {}
    hidden = ['location']
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map,
                                                           hidden)


class ListDMSInstance(command.Lister):
    _description = _('List DMS Instances')
    columns = ('ID', 'name', 'engine_name', 'engine_version',
               'storage_spec_code', 'status', 'connect_address', 'router_id',
               'network_id', 'security_group_id', 'user_name', 'storage',
               'total_storage', 'used_storage')

    def get_parser(self, prog_name):
        parser = super(ListDMSInstance, self).get_parser(prog_name)

        parser.add_argument(
            '--engine-name',
            metavar='<engine>',
            help=_('Engine name')
        )

        parser.add_argument(
            '--status',
            metavar='{' + ','.join(INSTANCE_STATUS_CHOICES) + '}',
            type=lambda s: s.upper(),
            choices=INSTANCE_STATUS_CHOICES,
            help=_('Instance status')
        )

        parser.add_argument(
            '--include-failure',
            action='store_true',
            help=_('Include instances failed to create')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dms

        query_params = {}
        for param in ['engine_name', 'status', 'include_failure']:
            val = getattr(parsed_args, param)
            if val is not None:
                query_params[param] = val

        data = client.instances(**query_params)

        table = (self.columns,
                 (utils.get_item_properties(
                     s, self.columns,
                 ) for s in data))
        return table


class ShowDMSInstance(command.ShowOne):
    _description = _('Show single Instance details')

    def get_parser(self, prog_name):
        parser = super(ShowDMSInstance, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            help=_('ID of the instance')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dms

        obj = client.find_instance(parsed_args.instance)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return (display_columns, data)


class DeleteDMSInstance(command.Command):
    _description = _('Delete DMS Instance')

    def get_parser(self, prog_name):
        parser = super(DeleteDMSInstance, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            nargs='+',
            help=_('ID of the Instance')
        )
        return parser

    def take_action(self, parsed_args):

        if parsed_args.instance:
            client = self.app.client_manager.dms
            for instance in parsed_args.instance:
                client.delete_instance(instance)


class CreateDMSInstance(command.ShowOne):
    _description = _('Create DMS Instance')

    def get_parser(self, prog_name):
        parser = super(CreateDMSInstance, self).get_parser(prog_name)
        parser.add_argument(
            'name',
            metavar='<name>',
            help=_('Name of the instance.')
        )

        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_('Description of the instance.')
        )
        parser.add_argument(
            '--engine-name',
            metavar='<engine>',
            help=_('Engine name. Currently only Kafka is supported.')
        )
        parser.add_argument(
            '--engine-version',
            metavar='<engine-ver>',
            help=_('Engine version. Currently only "2.3.0" is supported.')
        )
        parser.add_argument(
            '--storage',
            metavar='<GB>',
            type=int,
            required=True,
            help=_('Indicates the message storage space with increments '
                   'of 100 GB:\n'
                   'Instance with specification being 100MB: 600–90,000 GB\n'
                   'Instance with specification being 300MB: 1,200–90,000 GB\n'
                   'Instance with specification being 600MB: 2,400–90,000 GB\n'
                   'Instance with specification being 1200MB: 4,800–90,000 GB')
        )
        parser.add_argument(
            '--access-user',
            metavar='<access_user>',
            help=_(
                'This parameter is mandatory when engine is set to kafka and '
                'ssl_enable is set to true. This parameter is invalid when '
                'ssl_enable is set to false.\n'
                'Indicates a username. A username consists of 4 to 64 '
                'characters and supports only letters, digits, hyphens (-), '
                'and underscores (_).')
        )
        parser.add_argument(
            '--password',
            metavar='<password>',
            help=_(
                'This parameter is mandatory when engine is set to kafka and '
                'ssl_enable is set to true. This parameter is invalid when '
                'ssl_enable is set to false.\n'
                'An instance password must meet the following complexity '
                'requirements: \n'
                '- Must be a string consisting of 8 to 32 characters.\n'
                '- Must contain at least two of the following character '
                'types: \n'
                '-- Lowercase letters\n'
                '-- Uppercase letters\n'
                '-- Digits\n'
                '-- Special characters'
            )
        )
        parser.add_argument(
            '--router',
            metavar='<router>',
            required=True,
            help=_('Router ID or Name')
        )
        parser.add_argument(
            '--security-group',
            metavar='<sg>',
            required=True,
            help=_('Security group ID or Name')
        )
        parser.add_argument(
            '--network',
            metavar='<net>',
            required=True,
            help=_('Neutron network ID or Name')
        )
        parser.add_argument(
            '--availability-zone',
            metavar='<AZ>',
            required=True,
            action='append',
            help=_('List of availability zones')
        )
        parser.add_argument(
            '--product-id',
            metavar='<product>',
            required=True,
            help=_('Product ID of the DMS instance')
        )
        parser.add_argument(
            '--maintenance-begin',
            metavar='<HH24:MM>',
            help=_('Start of the instance maintenance window')
        )
        parser.add_argument(
            '--maintenance-end',
            metavar='<HH24:MM>',
            help=_('End of the instance maintenance window')
        )
        parser.add_argument(
            '--enable-public-access',
            action='store_true',
            help=_('Assign public ip to the instance')
        )
        parser.add_argument(
            '--enable-ssl',
            action='store_true',
            help=_('Enable SSL for the public access')
        )
        parser.add_argument(
            '--public-bandwidth',
            metavar='<Mbit/s>',
            type=int,
            help=_('Public network bandwidth in Mbit/s:\n'
                   'When specification 100MB: 3-900\n'
                   'When 300MB: 3-900\n'
                   'When 600MB: 4-1200\n'
                   'When 1200MB: 8-2400')
        )
        parser.add_argument(
            '--retention-policy',
            metavar='{' + ','.join(RETENTION_POLICY_CHOICES) + '}',
            type=lambda s: s.lower(),
            choices=RETENTION_POLICY_CHOICES,
            help=_('Action to be taken when the memory usage reaches the '
                   'disk capacity threshold. Options:\n'
                   ' `produce_reject`: New messages cannot be created.\n'
                   ' `time_base`: The earliest messages are deleted.')
        )
        parser.add_argument(
            '--storage-spec-code',
            metavar='{' + ','.join(STORAGE_SPEC_CHOICES) + '}',
            type=lambda s: s.lower(),
            choices=STORAGE_SPEC_CHOICES,
            help=_('The storage I/O specification of a Kafka instance.\n'
                   'When specification is 100MB, the storage I/O can be:'
                   '[`dms.physical.storage.high`, '
                   '`dms.physical.storage.ultra`]\n'
                   'When specification is 300MB, the storage I/O can be:'
                   '[`dms.physical.storage.high`, '
                   '`dms.physical.storage.ultra`]\n'
                   'When specification is 600MB, the storage I/O is '
                   '`dms.physical.storage.ultra`.\n'
                   'When specification is 1200MB, the storage I/O is '
                   '`dms.physical.storage.ultra`.')
        )

        return parser

    def take_action(self, parsed_args):

        attrs = {}

        attrs['name'] = parsed_args.name
        for attr in ['description', 'engine_name', 'engine_version', 'storage',
                     'access_user', 'password', 'product_id',
                     'maintenance_begin', 'maintenance_end',
                     'public_bandwidth',
                     'retention_policy', 'storage_spec_code']:
            val = getattr(parsed_args, attr)
            if val is not None:
                attrs[attr] = val

        network_client = self.app.client_manager.network

        router_obj = network_client.find_router(parsed_args.router,
                                                ignore_missing=False)
        attrs['router_id'] = router_obj.id
        net_obj = network_client.find_network(parsed_args.network,
                                              ignore_missing=False)
        attrs['network_id'] = net_obj.id
        sg_obj = self.app.client_manager.compute.find_security_group(
            parsed_args.security_group, ignore_missing=False)
        attrs['security_group_id'] = sg_obj.id

        if parsed_args.availability_zone:
            attrs['availability_zone'] = parsed_args.availability_zone

        if parsed_args.maintenance_begin and parsed_args.maintenance_end:
            attrs['maintenance_begin'] = parsed_args.maintenance_begin
            attrs['maintenance_end'] = parsed_args.maintenance_end
        elif parsed_args.maintenance_begin or parsed_args.maintenance_end:
            raise exceptions.CommandException(_(
                '`maintenance_start` and `maintenance_end` can be set only'
                'together'))
        if parsed_args.enable_public_access:
            attrs['is_public'] = True
            if parsed_args.enable_ssl:
                attrs['is_ssl'] = True

        client = self.app.client_manager.dms

        obj = client.create_instance(**attrs)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return (display_columns, data)


class UpdateDMSInstance(command.ShowOne):
    _description = _('Update DMS Instance')

    def get_parser(self, prog_name):
        parser = super(UpdateDMSInstance, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<inst>',
            help=_('Name or ID of the DMS instance')
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_('New name of the instance.')
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_('New description of the instance.')
        )
        parser.add_argument(
            '--security-group',
            metavar='<sg>',
            required=True,
            help=_('Security group ID or Name')
        )
        parser.add_argument(
            '--maintenance-begin',
            metavar='<HH24:MM>',
            help=_('Start of the instance maintenance window')
        )
        parser.add_argument(
            '--maintenance-end',
            metavar='<HH24:MM>',
            help=_('End of the instance maintenance window')
        )
        return parser

    def take_action(self, parsed_args):

        attrs = {}

        attrs['name'] = parsed_args.name
        for attr in ['description', 'maintenance_begin', 'maintenance_end']:
            val = getattr(parsed_args, attr)
            if val is not None:
                attrs[attr] = val

        sg_obj = self.app.client_manager.compute.find_security_group(
            parsed_args.security_group, ignore_missing=False)
        attrs['security_group_id'] = sg_obj.id

        if parsed_args.maintenance_begin and parsed_args.maintenance_end:
            attrs['maintenance_begin'] = parsed_args.maintenance_begin
            attrs['maintenance_end'] = parsed_args.maintenance_end
        elif parsed_args.maintenance_begin or parsed_args.maintenance_end:
            raise exceptions.CommandException(_(
                '`maintenance_start` and `maintenance_end` can be set only'
                'together'))

        client = self.app.client_manager.dms

        instance = client.find_instance(parsed_args.instance,
                                        ignore_missing=False)

        obj = client.update_instance(instance, **attrs)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return (display_columns, data)


class RestartDMSInstance(command.Command):
    _description = _('Restart single Instance')

    def get_parser(self, prog_name):
        parser = super(RestartDMSInstance, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            help=_('ID of the instance')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dms

        obj = client.find_instance(parsed_args.instance, ignore_missing=False)

        client.restart_instance(obj)

        return



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import _log

from otcextensions.tests.functional.sdk.auto_scaling.v1 import base

_logger = _log.setup_logging('openstack')


class TestQuota(base.TestAs):

    def setUp(self):
        super(TestQuota, self).setUp()
        self.auto_scaling = self.conn.auto_scaling

    def test_list(self):
        expected_types = ['scaling_Group', 'scaling_Config',
                          'scaling_Policy', 'scaling_Instance',
                          'load_balance', 'bandwidth_scaling_policy']
        objects = list(self.auto_scaling.quotas())

        self.assertGreaterEqual(len(objects), 6)
        types = []

        for obj in objects:
            types.append(obj.get('type'))
            self.assertIsInstance(obj, dict)
            self.assertIn('type', obj.keys())
            self.assertIn('used', obj.keys())
            self.assertIn('max', obj.keys())
            self.assertIn('quota', obj.keys())

        self.assertListEqual(types, expected_types)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import mock

from osc_lib.cli import format_columns

from otcextensions.osclient.rds.v3 import instance
from otcextensions.tests.unit.osclient.rds.v3 import fakes


class TestListDatabaseInstances(fakes.TestRds):

    objects = fakes.FakeInstance.create_multiple(3)

    column_list_headers = (
        'ID', 'Name', 'Datastore Type', 'Datastore Version', 'Status',
        'Flavor_ref', 'Type', 'Size', 'Region'
    )

    columns = (
        'id', 'name', 'datastore_type', 'datastore_version'
    )

    data = []

    for s in objects:
        data.append(
            (s.id, s.name, s.datastore['type'], s.datastore['version'],
             s.status, s.flavor_ref, s.type, s.volume['size'], s.region))

    def setUp(self):
        super(TestListDatabaseInstances, self).setUp()

        self.cmd = instance.ListDatabaseInstances(self.app, None)

        self.client.instances = mock.Mock()
        self.client.api_mock = self.client.instances

    def test_list(self):
        arglist = []

        verifylist = []

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with()

        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(self.data, list(data))

    def test_list_args(self):
        arglist = [
            '--limit', '1',
            '--id', '2',
            '--name', '3',
            '--type', 'ha',
            '--datastore-type', 'postgreSqL',
            '--router-id', '6',
            '--network-id', '7',
            '--offset', '8',
        ]

        verifylist = [
            ('limit', 1),
            ('id', '2'),
            ('name', '3'),
            ('type', 'ha'),
            ('datastore_type', 'postgresql'),
            ('router_id', '6'),
            ('network_id', '7'),
            ('offset', 8),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with(
            datastore_type='PostgreSQL',
            id='2',
            limit=1,
            name='3',
            offset=8,
            router_id='6',
            network_id='7',
            type='Ha',
            paginated=False
        )

    def test_list_args_paginated(self):
        arglist = [
            '--id', '2',
            '--name', '3',
            '--type', 'ha',
            '--datastore-type', 'postgreSqL',
            '--router-id', '6',
            '--network-id', '7',
            '--offset', '8',
        ]

        verifylist = [
            ('id', '2'),
            ('name', '3'),
            ('type', 'ha'),
            ('datastore_type', 'postgresql'),
            ('router_id', '6'),
            ('network_id', '7'),
            ('offset', 8),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with(
            datastore_type='PostgreSQL',
            id='2',
            name='3',
            offset=8,
            router_id='6',
            network_id='7',
            type='Ha',
        )


class TestShowDatabaseInstance(fakes.TestRds):

    _data = fakes.FakeInstance.create_one()

    columns = (
        'datastore', 'flavor_ref', 'id', 'name', 'region', 'status', 'volume'
    )

    data = (
        format_columns.DictColumn(_data.datastore),
        _data.flavor_ref,
        _data.id,
        _data.name,
        _data.region,
        _data.status,
        format_columns.DictColumn(_data.volume)
    )

    def setUp(self):
        super(TestShowDatabaseInstance, self).setUp()

        self.cmd = instance.ShowDatabaseInstance(self.app, None)

        self.client.find_instance = mock.Mock(return_value=self._data)

    def test_show(self):
        arglist = [
            'test_instance',
        ]

        verifylist = [
            ('instance', 'test_instance'),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)
        self.client.find_instance.assert_called_with('test_instance',
                                                     ignore_missing=False)

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestDeleteDatabaseInstance(fakes.TestRds):

    def setUp(self):
        super(TestDeleteDatabaseInstance, self).setUp()

        self.cmd = instance.DeleteDatabaseInstance(self.app, None)

        self.sdk_client.delete_instance = mock.Mock()

    def test_delete(self):
        arglist = [
            'test_obj',
        ]

        verifylist = [
            ('instance', 'test_obj'),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.sdk_client.delete_rds_instance.assert_called_with(
            instance='test_obj', wait=False)

    def test_delete_wait(self):
        arglist = [
            'test_obj',
            '--wait'
        ]

        verifylist = [
            ('instance', 'test_obj'),
            ('wait', True)
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.sdk_client.delete_rds_instance.assert_called_with(
            instance='test_obj', wait=True)


class TestCreateDatabaseInstance(fakes.TestRds):

    _data = fakes.FakeInstance.create_one()

    columns = ()

    data = fakes.gen_data(_data, columns)

    def setUp(self):
        super(TestCreateDatabaseInstance, self).setUp()

        self.cmd = instance.CreateDatabaseInstance(self.app, None)

        self.sdk_client.create_rds_instance = mock.Mock(
            return_value=self._data)

    def test_create(self):
        arglist = [
            'inst_name',
            'test-flavor',
            '--availability-zone', 'test-az-01',
            '--backup', 'source_backup',
            '--backup-keepdays', 'bkd',
            '--backup-timeframe', 'xxx',
            '--charge-mode', 'postpaid',
            '--configuration', '123',
            '--datastore-type', 'MySQL',
            '--datastore-version', '5.7',
            '--disk-encryption-id', '234',
            '--from-instance', 'source_instance',
            '--ha-mode', 'async',
            '--network-id', 'test-network-id',
            '--password', 'CrackMe123',
            '--port', '12345',
            '--region', 'test-region',
            '--replica-of', 'replica_instance',
            '--restore-time', 'restore_time_123',
            '--router-id', 'test-vpc-id',
            '--security-group-id', 'test-sec_grp-id',
            '--volume-type', 'ULTRAHIGH',
            '--size', '100'
        ]

        verifylist = [
            ('name', 'inst_name'),
            ('configuration', '123'),
            ('flavor', 'test-flavor'),
            ('availability_zone', 'test-az-01'),
            ('datastore_type', 'MySQL'),
            ('datastore_version', '5.7'),
            ('disk_encryption_id', '234'),
            ('router', 'test-vpc-id'),
            ('network', 'test-network-id'),
            ('security_group', 'test-sec_grp-id'),
            ('port', 12345),
            ('volume_type', 'ULTRAHIGH'),
            ('volume_size', 100),
            ('password', 'CrackMe123'),
            ('region', 'test-region')]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.sdk_client.create_rds_instance.assert_called_with(
            availability_zone='test-az-01', backup='source_backup',
            backup_keepdays='bkd', backup_timeframe='xxx',
            charge_mode='postpaid', configuration='123',
            datastore_type='MySQL', datastore_version='5.7',
            disk_encryption_id='234', flavor='test-flavor',
            from_instance='source_instance', ha_mode='async', name='inst_name',
            network='test-network-id', password='CrackMe123', port=12345,
            region='test-region', replica_of='replica_instance',
            restore_time='restore_time_123', router='test-vpc-id',
            security_group='test-sec_grp-id', volume_size=100,
            volume_type='ULTRAHIGH', wait=False
        )


class TestRestoreDatabaseInstance(fakes.TestRds):

    def setUp(self):
        super(TestRestoreDatabaseInstance, self).setUp()

        self.cmd = instance.RestoreDatabaseInstance(self.app, None)

        self.instance = self.instance_mock.create_one()
        self.backup = self.backup_mock.create_one()

        self.client.find_instance = mock.Mock(return_value=self.instance)
        self.restore_result = mock.Mock()
        self.restore_result.job_id = 'some_job_id'
        self.client.restore_instance = mock.Mock(
            return_value=self.restore_result)
        self.client.find_backup = mock.Mock(return_value=self.backup)
        self.client.wait_for_job = mock.Mock()

    def test_action_from_backup(self):
        arglist = [
            'inst_name',
            '--backup', 'backup',
        ]

        verifylist = [
            ('instance', 'inst_name'),
            ('backup', 'backup'),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.client.find_instance.assert_called_with('inst_name',
                                                     ignore_missing=False)

        self.client.find_backup.assert_called_with(name_or_id='backup',
                                                   instance=self.instance,
                                                   ignore_missing=False)

        self.client.restore_instance.assert_called_with(
            backup=self.client.find_backup(),
            instance=self.instance,
            restore_time=None
        )

    def test_action_pit(self):
        arglist = [
            'inst_name',
            '--restore-time', 'some_time',
        ]

        verifylist = [
            ('instance', 'inst_name'),
            ('restore_time', 'some_time'),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.client.find_instance.assert_called_with('inst_name',
                                                     ignore_missing=False)

        self.client.restore_instance.assert_called_with(
            backup=None,
            instance=self.instance,
            restore_time='some_time'
        )

    def test_action_wait(self):
        arglist = [
            'inst_name',
            '--restore-time', 'some_time',
            '--wait',
            '--wait-interval', '5'
        ]

        verifylist = [
            ('instance', 'inst_name'),
            ('restore_time', 'some_time'),
            ('wait', True),
            ('wait_interval', 5)
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.client.find_instance.assert_called_with('inst_name',
                                                     ignore_missing=False)

        self.client.restore_instance.assert_called_with(
            backup=None,
            instance=self.instance,
            restore_time='some_time'
        )

        self.client.wait_for_job.assert_called_with(
            self.restore_result.job_id,
            interval=5
        )


class TestShowBackupPolicy(fakes.TestRds):

    _instance = fakes.FakeInstance.create_one()
    _data = {'keep_days': 1, 'period': '2', 'start_time': '3'}

    columns = (
        'keep_days', 'period', 'start_time'
    )

    data = (1, '2', '3')

    def setUp(self):
        super(TestShowBackupPolicy, self).setUp()

        self.cmd = instance.ShowBackupPolicy(self.app, None)

        self.client.find_instance = mock.Mock()
        self.client.get_instance_backup_policy = mock.Mock()

    def test_action(self):
        arglist = [
            'inst'
        ]

        verifylist = [
            ('instance', 'inst'),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.find_instance.return_value = self._instance
        self.client.get_instance_backup_policy.return_value = self._data

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_instance.assert_called_with('inst',
                                                     ignore_missing=False)

        self.client.get_instance_backup_policy.assert_called_once_with(
            self._instance
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestSetBackupPolicy(fakes.TestRds):

    _instance = fakes.FakeInstance.create_one()

    def setUp(self):
        super(TestSetBackupPolicy, self).setUp()

        self.cmd = instance.SetBackupPolicy(self.app, None)

        self.client.find_instance = mock.Mock()
        self.client.set_instance_backup_policy = mock.Mock()

    def test_action(self):
        arglist = [
            'inst',
            '--keep-days', '1',
            '--period', '2',
            '--start-time', '3'
        ]

        verifylist = [
            ('instance', 'inst'),
            ('keep_days', 1),
            ('period', '2'),
            ('start_time', '3')
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.find_instance.return_value = self._instance

        # Trigger the action
        self.assertIsNone(self.cmd.take_action(parsed_args))

        self.client.find_instance.assert_called_with('inst',
                                                     ignore_missing=False)

        self.client.set_instance_backup_policy.assert_called_once_with(
            self._instance,
            keep_days=1, period='2', start_time='3'
        )

    def test_action_set_0(self):
        arglist = [
            'inst',
            '--keep-days', '0'
        ]

        verifylist = [
            ('instance', 'inst'),
            ('keep_days', 0),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.find_instance.return_value = self._instance

        # Trigger the action
        self.assertIsNone(self.cmd.take_action(parsed_args))

        self.client.find_instance.assert_called_with('inst',
                                                     ignore_missing=False)

        self.client.set_instance_backup_policy.assert_called_once_with(
            self._instance,
            keep_days=0
        )


class TestShowAvailableRestoreTime(fakes.TestRds):

    instance = fakes.FakeInstance.create_one()

    objects = [
        {
            'start_time': 'some_fake_start',
            'end_time': 'some_fake_end'
        }
    ]

    column_list_headers = (
        'Start time', 'End time'
    )

    columns = (
        'start_time', 'end_time'
    )

    data = []

    for s in objects:
        data.append((s['start_time'], s['end_time']))

    def setUp(self):
        super(TestShowAvailableRestoreTime, self).setUp()

        self.cmd = instance.ShowAvailableRestoreTime(self.app, None)

        self.client.find_instance = mock.Mock(return_value=self.instance)
        self.client.get_instance_restore_time = mock.Mock(
            return_value=self.objects)

    def test_action(self):
        arglist = [
            'test_inst'
        ]

        verifylist = [
            ('instance', 'test_inst')
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_instance.assert_called_with('test_inst',
                                                     ignore_missing=False)
        self.client.get_instance_restore_time.assert_called_with(self.instance)

        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(self.data, list(data))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class Flavor(resource.Resource):

    base_path = '/flavors/%(datastore_name)s'
    resources_key = 'flavors'

    # capabilities
    allow_list = True
    _query_mapping = resource.QueryParameters(
        'version_name', 'spec_code')

    datastore_name = resource.URI('datastore_name')
    #: Instance Mode (single/ha/replica)
    #: *Type: int*
    instance_mode = resource.Body('instance_mode')
    #: Ram size in MB.
    #: *Type:int*
    ram = resource.Body('ram', type=int)
    #: Flavor name.
    #: *Type:str*
    name = resource.Body('name', alias='spec_code')
    #: Specification code
    #: *Type: str*
    spec_code = resource.Body('spec_code')
    #: Amount of VCPU's
    #: *Type: str*
    vcpus = resource.Body('vcpus')
    #: Group type
    #: *Type: str*
    group_type = resource.Body('group_type')
    #: Supported Versions
    #: *Type:list*
    version_name = resource.Body('version_name', type=list)
    #: Availability Zone Status
    #: *Type: dict*
    az_status = resource.Body('az_status', type=dict)
    #: Availability Zone Description
    #: *Type: dict*
    az_desc = resource.Body('az_desc', type=dict)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base

from otcextensions.sdk.function_graph.v2 import template


EXAMPLE = {
    "id": "41d5d9ca-cea3-4ba9-b866-e30c46f45f1f",
    "type": 1,
    "title": "access-mysql",
    "template_name": "access-mysql-js-12.13",
    "runtime": "Node.js12.13",
    "handler": "index.handler",
    "code_type": "",
    "code": "",
    "timeout": 30,
    "memory_size": 256,
    "temp_detail": {
        "input": "None",
        "output": "Execution successful: Database query results",
        "warning": ""
    },
    "scene": "basic_function_usage",
    "service": "FunctionGraph"
}


class TestFunctionInvocation(base.TestCase):

    def test_basic(self):
        sot = template.Template()
        path = 'fgs/templates/%(template_id)s'
        self.assertEqual(path, sot.base_path)
        self.assertTrue(sot.allow_fetch)

    def test_make_it(self):
        sot = template.Template(**EXAMPLE)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['type'], sot.type)
        self.assertEqual(EXAMPLE['title'], sot.title)
        self.assertEqual(EXAMPLE['template_name'], sot.template_name)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class Version(resource.Resource):
    base_path = '/fgs/functions/%(function_urn)s/versions'
    resources_key = 'versions'
    # Capabilities
    allow_create = True
    allow_fetch = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'maxitems', 'marker'
    )

    # Properties
    function_urn = resource.URI('function_urn', type=str)
    digest = resource.Body('digest', type=str)
    version = resource.Body('version', type=str)
    description = resource.Body('description', type=str)

    # Attributes
    next_marker = resource.Body('next_marker', type=int)
    count = resource.Body('count', type=int)
    func_urn = resource.Body('func_urn', type=str)
    func_name = resource.Body('func_name', type=str)
    domain_id = resource.Body('domain_id', type=str)
    project_id = resource.Body('namespace', type=str)
    project_name = resource.Body('project_name', type=str)
    package = resource.Body('package', type=str)
    runtime = resource.Body('runtime', type=str)
    timeout = resource.Body('timeout', type=int)
    handler = resource.Body('handler', type=str)
    memory_size = resource.Body('memory_size', type=int)
    cpu = resource.Body('cpu', type=int)
    code_type = resource.Body('code_type', type=str)
    code_url = resource.Body('code_url', type=str)
    code_filename = resource.Body('code_filename', type=str)
    code_size = resource.Body('code_size', type=int)
    user_data = resource.Body('user_data', type=str)
    encrypted_user_data = resource.Body('encrypted_user_data', type=str)
    image_name = resource.Body('image_name', type=str)
    xrole = resource.Body('xrole', type=str)
    app_xrole = resource.Body('app_xrole', type=str)
    version_description = resource.Body('version_description', type=str)
    last_modified = resource.Body('last_modified', type=str)
    func_vpc = resource.Body('func_vpc', type=dict)
    mount_config = resource.Body('mount_config', type=dict)
    strategy_config = resource.Body('strategy_config', type=dict)
    dependencies = resource.Body('dependencies', type=dict)
    initializer_handler = resource.Body('initializer_handler', type=str)
    initializer_timeout = resource.Body('initializer_timeout', type=int)
    pre_stop_handler = resource.Body('pre_stop_handler', type=str)
    pre_stop_timeout = resource.Body('pre_stop_timeout', type=int)
    enterprise_project_id = resource.Body('enterprise_project_id', type=str)
    long_time = resource.Body('long_time', type=bool)
    log_group_id = resource.Body('log_group_id', type=str)
    log_stream_id = resource.Body('log_stream_id', type=str)
    type = resource.Body('type', type=str)
    enable_dynamic_memory = resource.Body('enable_dynamic_memory', type=bool)
    function_async_config = resource.Body('function_async_config', type=dict)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from otcextensions.tests.functional.sdk.apig import TestApiG


class TestBackendServer(TestApiG):
    gateway = "560de602c9f74969a05ff01d401a53ed"
    vpc = "80fc1bd30db94ef49468a27f536dd2cd"
    group = "vpc_member_group"
    server = None

    def setUp(self):
        super(TestBackendServer, self).setUp()
        attrs = {
            "members": [{
                "host": "192.168.2.25",
                "weight": 1,
                "member_group_name": "vpc_member_group"
            }]
        }
        result = self.client.add_or_update_backend_servers(
            gateway=TestBackendServer.gateway,
            vpc_channel=TestBackendServer.vpc,
            **attrs
        )
        TestBackendServer.server = result[0]
        self.assertIsNotNone(result[0].id)

    def tearDown(self):
        super(TestBackendServer, self).tearDown()
        self.client.remove_backend_server(
            gateway=TestBackendServer.gateway,
            vpc_channel=TestBackendServer.vpc,
            backend_server=TestBackendServer.server.id
        )

    def test_01_list_backend_servers(self):
        found = list(self.client.list_backend_servers(
            gateway=TestBackendServer.gateway,
            vpc_channel=TestBackendServer.vpc,
        ))
        self.assertEqual(1, len(found))

    def test_02_update_backend_server(self):
        attrs = {
            "member_group_name": "vpc_member_group",
            "members": [{
                "host": "192.168.2.25",
                "weight": 2,
                "is_backup": True,
                "member_group_name": "vpc_member_group"
            }]
        }
        updated = self.client.update_backend_server(
            gateway=TestBackendServer.gateway,
            vpc_channel=TestBackendServer.vpc,
            **attrs
        )
        self.assertEqual(updated[0].is_backup, True)

    def test_03_enable_backend_server(self):
        attrs = {
            "member_ids": [TestBackendServer.server.id]
        }
        self.client.enable_backend_server(
            gateway=TestBackendServer.gateway,
            vpc_channel=TestBackendServer.vpc,
            backend_server=TestBackendServer.server.id,
            **attrs
        )

    def test_04_disable_backend_server(self):
        attrs = {
            "member_ids": [TestBackendServer.server.id]
        }
        self.client.disable_backend_server(
            gateway=TestBackendServer.gateway,
            vpc_channel=TestBackendServer.vpc,
            backend_server=TestBackendServer.server.id,
            **attrs
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import base64
import os

from openstack.tests.unit import test_proxy_base
from otcextensions.sdk.modelartsv2.v2 import _proxy
from otcextensions.sdk.modelartsv2.v2 import dataset
from otcextensions.sdk.modelartsv2.v2 import dataset_export_task
from otcextensions.sdk.modelartsv2.v2 import dataset_import_task
from otcextensions.sdk.modelartsv2.v2 import dataset_label
from otcextensions.sdk.modelartsv2.v2 import dataset_sample
from otcextensions.sdk.modelartsv2.v2 import dataset_statistics
from otcextensions.sdk.modelartsv2.v2 import dataset_sync
from otcextensions.sdk.modelartsv2.v2 import dataset_version


class TestModelartsV2Proxy(test_proxy_base.TestProxyBase):
    def setUp(self):
        super(TestModelartsV2Proxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)


class TestDataset(TestModelartsV2Proxy):
    def test_datasets(self):
        self.verify_list(
            self.proxy.datasets,
            dataset.Dataset,
            method_kwargs={"limit": 10},
            expected_kwargs={"limit": 10},
        )

    def test_get_dataset(self):
        self.verify_get(
            self.proxy.get_dataset,
            dataset.Dataset,
            method_args=["dataset-uuid"],
            expected_args=["dataset-uuid"],
        )

    def test_find_dataset(self):
        self.verify_find(
            self.proxy.find_dataset,
            dataset.Dataset,
        )

    def test_create_dataset(self):
        self.verify_create(
            self.proxy.create_dataset,
            dataset.Dataset,
            method_kwargs={"arg1": "val1", "arg2": "val2"},
            expected_kwargs={"arg1": "val1", "arg2": "val2"},
        )

    def test_delete_dataset(self):
        self.verify_delete(
            self.proxy.delete_dataset,
            dataset.Dataset,
            False,
        )

    def test_delete_dataset_ignore(self):
        self.verify_delete(
            self.proxy.delete_dataset,
            dataset.Dataset,
            True,
        )


class TestDatasetSample(TestModelartsV2Proxy):
    def test_dataset_samples(self):
        self.verify_list(
            self.proxy.dataset_samples,
            dataset_sample.DatasetSample,
            method_args=["dataset-uuid"],
            expected_args=[],
            method_kwargs={"limit": 10},
            expected_kwargs={"uri_dataset_id": "dataset-uuid", "limit": 10},
        )

    def test_get_dataset_sample(self):
        self.verify_get(
            self.proxy.get_dataset_sample,
            dataset_sample.DatasetSample,
            method_args=["dataset-uuid", "sample-uuid"],
            expected_args=["sample-uuid"],
            expected_kwargs={"uri_dataset_id": "dataset-uuid"},
        )

    def test_add_dataset_samples(self):
        dirname = os.path.dirname(__file__)
        file_path = os.path.join(dirname, "8710109684_e2c5ef6aeb_n.jpg")
        with open(file_path, "rb") as file:
            sample = {
                "name": "8710109684_e2c5ef6aeb_n.jpg",
                "data": base64.b64encode(file.read()).decode(),
            }
        self.verify_create(
            self.proxy.add_dataset_samples,
            dataset_sample.DatasetSample,
            method_args=["dataset-uuid"],
            expected_args=[],
            method_kwargs={
                "file_path": file_path,
            },
            expected_kwargs={
                "uri_dataset_id": "dataset-uuid",
                "samples": [sample],
            },
        )

    def test_delete_dataset_samples(self):
        self._verify(
            "otcextensions.sdk.modelartsv2.v2.dataset_sample.DatasetSample.delete_samples",  # noqa
            self.proxy.delete_dataset_samples,
            method_args=["dataset-id"],
            method_kwargs={
                "samples": ["s1-uuid", "s2-uuid"],
                "delete_source": False,
            },
            expected_args=[
                self.proxy,
                "dataset-id",
                ["s1-uuid", "s2-uuid"],
                False,
            ],
        )


class TestDatasetLabel(TestModelartsV2Proxy):
    def test_dataset_labels(self):
        self.verify_list(
            self.proxy.dataset_labels,
            dataset_label.DatasetLabel,
            method_kwargs={"dataset": "dataset-id"},
            expected_kwargs={"uri_dataset_id": "dataset-id"},
        )

    def test_create_dataset_label(self):
        self.verify_create(
            self.proxy.create_dataset_label,
            dataset_label.DatasetLabel,
            method_kwargs={
                "dataset": "dataset-id",
                "arg1": "val1",
                "arg2": "val2",
            },
            expected_kwargs={
                "uri_dataset_id": "dataset-id",
                "arg1": "val1",
                "arg2": "val2",
            },
        )

    def test_update_dataset_labels(self):
        self._verify(
            "otcextensions.sdk.modelartsv2.v2.dataset_label.DatasetLabel.update_labels",  # noqa: E501
            self.proxy.update_dataset_labels,
            method_args=["dataset-id"],
            method_kwargs={
                "labels": [{"name": "cat"}, {"name": "pussycat"}],
            },
            expected_args=[
                self.proxy,
                [{"name": "cat"}, {"name": "pussycat"}],
            ],
        )

    def test_delete_dataset_labels(self):
        self._verify(
            "otcextensions.sdk.modelartsv2.v2.dataset_label.DatasetLabel.delete_labels",  # noqa: E501
            self.proxy.delete_dataset_labels,
            method_args=["dataset-id"],
            method_kwargs={
                "labels": [{"name": "cat"}, {"name": "pussycat"}],
                "delete_policy": 0,
            },
            expected_args=[
                self.proxy,
                [{"name": "cat"}, {"name": "pussycat"}],
                0,
            ],
        )


class TestDatasetExportTask(TestModelartsV2Proxy):
    def test_dataset_export_tasks(self):
        self.verify_list(
            self.proxy.dataset_export_tasks,
            dataset_export_task.DatasetExportTask,
            method_kwargs={"dataset": "dataset-id"},
            expected_kwargs={"uri_dataset_id": "dataset-id"},
        )

    def test_get_dataset_export_task(self):
        self.verify_get(
            self.proxy.get_dataset_export_task,
            dataset_export_task.DatasetExportTask,
            method_args=["dataset-id", "task-id"],
            expected_args=["task-id"],
            expected_kwargs={"uri_dataset_id": "dataset-id"},
        )

    def test_create_dataset_export_task(self):
        self.verify_create(
            self.proxy.create_dataset_export_task,
            dataset_export_task.DatasetExportTask,
            method_kwargs={"dataset": "dataset-id"},
            expected_kwargs={"uri_dataset_id": "dataset-id"},
        )


class TestDatasetImportTask(TestModelartsV2Proxy):
    def test_dataset_import_tasks(self):
        self.verify_list(
            self.proxy.dataset_import_tasks,
            dataset_import_task.DatasetImportTask,
            method_kwargs={"dataset": "dataset-id"},
            expected_kwargs={"uri_dataset_id": "dataset-id"},
        )

    def test_get_dataset_export_task(self):
        self.verify_get(
            self.proxy.get_dataset_import_task,
            dataset_import_task.DatasetImportTask,
            method_args=["dataset-id", "task-id"],
            expected_args=["task-id"],
            expected_kwargs={"uri_dataset_id": "dataset-id"},
        )

    def test_create_dataset_import_task(self):
        self.verify_create(
            self.proxy.create_dataset_import_task,
            dataset_import_task.DatasetImportTask,
            method_kwargs={"dataset": "dataset-id"},
            expected_kwargs={"uri_dataset_id": "dataset-id"},
        )


class TestDatasetVersion(TestModelartsV2Proxy):
    def test_dataset_versions(self):
        self.verify_list(
            self.proxy.dataset_versions,
            dataset_version.DatasetVersion,
            method_kwargs={"dataset": "dataset-id"},
            expected_kwargs={"uri_dataset_id": "dataset-id"},
        )

    def test_get_dataset_version(self):
        self.verify_get(
            self.proxy.get_dataset_version,
            dataset_version.DatasetVersion,
            method_args=["dataset-id", "version-id"],
            expected_args=["version-id"],
            expected_kwargs={"uri_dataset_id": "dataset-id"},
        )

    def test_create_dataset_version(self):
        self.verify_create(
            self.proxy.create_dataset_version,
            dataset_version.DatasetVersion,
            method_kwargs={"dataset": "dataset-id"},
            expected_kwargs={"uri_dataset_id": "dataset-id"},
        )

    def test_delete_dataset_version(self):
        self.verify_delete(
            self.proxy.delete_dataset_version,
            dataset_version.DatasetVersion,
            False,
            method_args=["dataset-id", "version-id"],
            expected_args=["version-id"],
            expected_kwargs={"uri_dataset_id": "dataset-id"},
        )


class TestDatasetStatistics(TestModelartsV2Proxy):
    def test_get_dataset_statistics(self):
        self.verify_get(
            self.proxy.get_dataset_statistics,
            dataset_statistics.DatasetStatistics,
            method_args=[],
            method_kwargs={"dataset": "dataset-id"},
            expected_kwargs={
                "uri_dataset_id": "dataset-id",
                "requires_id": False
            },
        )


class TestDatasetSync(TestModelartsV2Proxy):
    def test_get_dataset_sync_status(self):
        self.verify_get(
            self.proxy.get_dataset_sync_status,
            dataset_sync.DatasetSync,
            method_args=[],
            expected_args=["status"],
            method_kwargs={"dataset": "dataset-id"},
            expected_kwargs={"uri_dataset_id": "dataset-id"},
        )

    def test_dataset_sync(self):
        self.verify_create(
            self.proxy.dataset_sync,
            dataset_sync.DatasetSync,
            method_kwargs={"dataset": "dataset-id"},
            expected_kwargs={"uri_dataset_id": "dataset-id"},
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from unittest import mock

from otcextensions.sdk.function_graph.v2 import _proxy
from otcextensions.sdk.function_graph.v2 import function as _function
from otcextensions.sdk.function_graph.v2 import function_invocation as _fi
from otcextensions.sdk.function_graph.v2 import dependency as _d
from otcextensions.sdk.function_graph.v2 import quota as _q
from otcextensions.sdk.function_graph.v2 import event as _event
from otcextensions.sdk.function_graph.v2 import alias as _alias
from otcextensions.sdk.function_graph.v2 import version as _version
from otcextensions.sdk.function_graph.v2 import metric as _metric
from otcextensions.sdk.function_graph.v2 import log as _log
from otcextensions.sdk.function_graph.v2 import template as _t
from otcextensions.sdk.function_graph.v2 import reserved_instance as _r
from otcextensions.sdk.function_graph.v2 import import_function as _import
from otcextensions.sdk.function_graph.v2 import trigger as _trigger
from otcextensions.sdk.function_graph.v2 import async_notification as _async
from openstack.tests.unit import test_proxy_base


class TestFgProxy(test_proxy_base.TestProxyBase):
    def setUp(self):
        super(TestFgProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)


class TestFgFunctions(TestFgProxy):

    def test_functions(self):
        self.verify_list(self.proxy.functions, _function.Function)

    def test_create_function(self):
        self.verify_create(self.proxy.create_function,
                           _function.Function,
                           method_kwargs={'name': 'test_function'},
                           expected_kwargs={'name': 'test_function'})

    def test_delete_function(self):
        function = _function.Function(
            func_urn='urn:fss:eu-de:45c274f200d2498683982c8741fb76ac:'
                     'function:default'
                     ':access-mysql-js-1213-1737554083545:latest')
        self.proxy._get_resource = mock.Mock(return_value=function)
        self._verify(
            'otcextensions.sdk.function_graph.v2.function.'
            'Function._delete_function',
            self.proxy.delete_function,
            method_args=[function],
            expected_args=[self.proxy, function],
        )

    def test_get_function_code(self):
        function = _function.Function(
            func_urn='urn:fss:eu-de:45c274f200d2498683982c8741fb76ac:'
                     'function:default'
                     ':access-mysql-js-1213-1737554083545:latest')
        self.proxy._get_resource = mock.Mock(return_value=function)
        self._verify(
            'otcextensions.sdk.function_graph.v2.function.'
            'Function._get_function_code',
            self.proxy.get_function_code,
            method_args=[function],
            expected_args=[self.proxy, function],
        )

    def test_get_function_metadata(self):
        function = _function.Function(
            func_urn='urn:fss:eu-de:45c274f200d2498683982c8741fb76ac:'
                     'function:default'
                     ':access-mysql-js-1213-1737554083545:latest')
        self.proxy._get_resource = mock.Mock(return_value=function)
        self._verify(
            'otcextensions.sdk.function_graph.v2.function.'
            'Function._get_function_metadata',
            self.proxy.get_function_metadata,
            method_args=[function],
            expected_args=[self.proxy, function]
        )

    def test_get_resource_tags(self):
        function = _function.Function(id='test_function')
        self._verify(
            'otcextensions.sdk.function_graph.v2.function.'
            'Function._get_resource_tags',
            self.proxy.get_resource_tags,
            method_args=[function],
            expected_args=[self.proxy, function]
        )

    def test_create_resource_tags(self):
        function = _function.Function(id='test_function')
        tags = [{'key': 'tag1', 'value': 'value1'}]
        self._verify(
            'otcextensions.sdk.function_graph.v2.function.'
            'Function._create_resource_tags',
            self.proxy.create_resource_tags,
            method_args=[function, tags],
            expected_args=[self.proxy, function, tags]
        )

    def test_delete_resource_tags(self):
        function = _function.Function(id='test_function')
        tags = [{'key': 'tag1'}]
        self._verify(
            'otcextensions.sdk.function_graph.v2.function.'
            'Function._delete_resource_tags',
            self.proxy.delete_resource_tags,
            method_args=[function, tags],
            expected_args=[self.proxy, function, tags]
        )

    def test_update_pin_status(self):
        function = _function.Function(id='test_function')
        self._verify(
            'otcextensions.sdk.function_graph.v2.function.'
            'Function._update_pin_status',
            self.proxy.update_pin_status,
            method_args=[function],
            expected_args=[self.proxy, function]
        )

    def test_update_function_code(self):
        function = _function.Function(id='test_function')
        attrs = {
            'code_type': 'zip',
            'code_url': 'https://example.com/code.zip',
            'code_filename': 'code.zip',
            'func_code': {'file': 'content'}
        }
        self._verify(
            'otcextensions.sdk.function_graph.v2.function.'
            'Function._update_function_code',
            self.proxy.update_function_code,
            method_args=[function],
            method_kwargs=attrs,
            expected_args=[self.proxy, function],
            expected_kwargs=attrs
        )

    def test_update_function_metadata(self):
        function = _function.Function(id='test_function')
        attrs = {'description': 'Updated metadata'}
        self._verify(
            'otcextensions.sdk.function_graph.v2.function.'
            'Function._update_function_metadata',
            self.proxy.update_function_metadata,
            method_args=[function],
            method_kwargs=attrs,
            expected_args=[self.proxy, function],
            expected_kwargs=attrs
        )

    def test_update_max_instances(self):
        function = _function.Function(id='test_function')
        instances = 10
        self._verify(
            'otcextensions.sdk.function_graph.v2.function.'
            'Function._update_max_instances',
            self.proxy.update_max_instances,
            method_args=[function, instances],
            expected_args=[self.proxy, function, 10],
        )

    def test_invocation_async(self):
        fi = _fi.FunctionInvocation(
            func_urn='urn:fss:eu-de:45c274f200d2498683982c8741fb76ac:'
                     'function:default'
                     ':access-mysql-js-1213-1737554083545:latest',
            attrs={'a': 'b'}
        )
        self.proxy._get_resource = mock.Mock(return_value=fi)
        self._verify(
            'otcextensions.sdk.function_graph.v2.function_invocation.'
            'FunctionInvocation._invocation',
            self.proxy.executing_function_asynchronously,
            method_args=[fi.func_urn],
            method_kwargs={'a': 'b'},
            expected_args=[self.proxy, 'invocations-async'],
            expected_kwargs={'a': 'b'}
        )

    def test_invocation_sync(self):
        fi = _fi.FunctionInvocation(
            func_urn='urn:fss:eu-de:45c274f200d2498683982c8741fb76ac:'
                     'function:default'
                     ':access-mysql-js-1213-1737554083545:latest',
            attrs={'a': 'b'}
        )
        self.proxy._get_resource = mock.Mock(return_value=fi)
        self._verify(
            'otcextensions.sdk.function_graph.v2.function_invocation.'
            'FunctionInvocation._invocation',
            self.proxy.executing_function_synchronously,
            method_args=[fi.func_urn],
            method_kwargs={'a': 'b'},
            expected_args=[self.proxy, 'invocations'],
            expected_kwargs={'a': 'b'}
        )


class TestFgQuotas(TestFgProxy):
    def test_quotas(self):
        self.verify_list(self.proxy.quotas, _q.Quota)


class TestFgDependencies(TestFgProxy):
    def test_dependencies(self):
        self.verify_list(
            self.proxy.dependencies,
            _d.Dependency,
            expected_args=[],
        )

    def test_create_dependency_version(self):
        self.verify_create(
            self.proxy.create_dependency_version,
            _d.Dependency
        )

    def test_delete_dependency_version(self):
        dep = _d.Dependency(
            version=1,
            dep_id='edbd67fa-f107-40b3-af75-a85f0577ad61'
        )
        self.proxy._get_resource = mock.Mock(return_value=dep)
        self._verify(
            'otcextensions.sdk.function_graph.v2.dependency.'
            'Dependency._delete_version',
            self.proxy.delete_dependency_version,
            method_args=[dep],
            expected_args=[self.proxy, dep],
        )

    def test_dependency_versions(self):
        dep = _d.Dependency(
            version=1,
            dep_id='edbd67fa-f107-40b3-af75-a85f0577ad61'
        )
        self.verify_list(
            self.proxy.dependency_versions,
            _d.Dependency,
            method_args=[dep],
            expected_args=[],
        )

    def test_get_dependency_version(self):
        dep = _d.Dependency(
            version=1,
            dep_id='edbd67fa-f107-40b3-af75-a85f0577ad61'
        )
        self.verify_get(
            self.proxy.get_dependency_version,
            _d.Dependency,
            method_args=[dep],
            expected_args=[],
            expected_kwargs={"requires_id": False}
        )


class TestFgEvents(TestFgProxy):
    def test_events(self):
        func_urn = ('urn:fss:eu-de:45c274f200d2498683982c8741fb76ac:'
                    'function:default:access-mysql-js-1213-1737554083545:'
                    'latest')
        self.verify_list(
            self.proxy.events,
            _event.Event,
            method_args=[func_urn],
            expected_args=[],
            expected_kwargs={
                'function_urn': func_urn.rpartition(":")[0]
            },
        )

    def test_create_event(self):
        func = _function.Function(
            name='test',
            func_urn='urn:fss:eu-de:45c274f200d2498683982c8741fb76ac:'
                     'function:default:access-mysql-js-1213-1737554083545:'
                     'latest'
        )
        self.verify_create(
            self.proxy.create_event,
            _event.Event,
            method_kwargs={
                'function': func,
                'name': 'test_event'
            },
            expected_kwargs={
                'function_urn': func.func_urn.rpartition(":")[0],
                'name': 'test_event'}
        )

    def test_delete_event(self):
        func = _function.Function(
            name='test',
            func_urn='urn:fss:eu-de:45c274f200d2498683982c8741fb76ac:'
                     'function:default:access-mysql-js-1213-1737554083545:'
                     'latest'
        )
        event = _event.Event(
            name='test',
        )
        self.verify_delete(
            self.proxy.delete_event,
            _event.Event,
            method_args=[func, event],
            expected_args=[event],
            expected_kwargs={'function_urn': func.func_urn.rpartition(":")[0]}
        )

    def test_update_event(self):
        func = _function.Function(
            name='test',
            func_urn='urn:fss:eu-de:45c274f200d2498683982c8741fb76ac:'
                     'function:default:access-mysql-js-1213-1737554083545:'
                     'latest'
        )
        event = _event.Event(name='test')
        self.verify_update(
            self.proxy.update_event,
            _event.Event,
            method_args=[func, event],
            expected_args=[event],
            expected_kwargs={
                'function_urn': func.func_urn.rpartition(":")[0],
                'x': 1,
                'y': 2,
                'z': 3
            }
        )

    def test_get_event(self):
        func = _function.Function(
            name='test',
            func_urn='urn:fss:eu-de:45c274f200d2498683982c8741fb76ac:'
                     'function:default:access-mysql-js-1213-1737554083545:'
                     'latest'
        )
        event = _event.Event(name='test')
        self.verify_get(
            self.proxy.get_event,
            _event.Event,
            method_args=[func, event],
            expected_args=[event],
            expected_kwargs={
                'function_urn': func.func_urn,
            }
        )


class TestFgAlias(TestFgProxy):
    def test_aliases(self):
        func_urn = ('urn:fss:eu-de:45c274f200d2498683982c8741fb76ac:'
                    'function:default:access-mysql-js-1213-1737554083545:'
                    'latest')
        self.verify_list(
            self.proxy.aliases,
            _alias.Alias,
            method_args=[func_urn],
            expected_args=[],
            expected_kwargs={
                'function_urn': func_urn.rpartition(":")[0]
            },
        )

    def test_create_alias(self):
        func = _function.Function(
            name='test',
            id='urn:fss:eu-de:45c274f200d2498683982c8741fb76ac:'
               'function:default:access-mysql-js-1213-1737554083545:'
               'latest'
        )
        self.verify_create(
            self.proxy.create_alias,
            _alias.Alias,
            method_kwargs={
                'function': func,
                'name': 'test_event'
            },
            expected_kwargs={
                'function_urn': func.id.rpartition(":")[0],
                'name': 'test_event'}
        )

    def test_delete_alias(self):
        func = _function.Function(
            name='test',
            func_urn='urn:fss:eu-de:45c274f200d2498683982c8741fb76ac:'
                     'function:default:access-mysql-js-1213-1737554083545:'
                     'latest'
        )
        a = _alias.Alias(
            name='test',
        )
        self.verify_delete(
            self.proxy.delete_alias,
            _alias.Alias,
            method_args=[func, a],
            expected_args=[a],
            expected_kwargs={'function_urn': func.func_urn.rpartition(":")[0]}
        )

    def test_update_event(self):
        func = _function.Function(
            name='test',
            func_urn='urn:fss:eu-de:45c274f200d2498683982c8741fb76ac:'
                     'function:default:access-mysql-js-1213-1737554083545:'
                     'latest'
        )
        a = _alias.Alias(name='test')
        attrs = {
            'version': 'new-version',
            'description': 'new',
        }
        self._verify(
            'otcextensions.sdk.function_graph.v2.alias.'
            'Alias._update_alias',
            self.proxy.update_alias,
            method_args=[func, a],
            method_kwargs=attrs,
            expected_args=[self.proxy, func, a],
            expected_kwargs=attrs
        )

    def test_get_event(self):
        func = _function.Function(
            name='test',
            func_urn='urn:fss:eu-de:45c274f200d2498683982c8741fb76ac:'
                     'function:default:access-mysql-js-1213-1737554083545:'
                     'latest'
        )
        a = _alias.Alias(name='test')
        self.verify_get(
            self.proxy.get_alias,
            _alias.Alias,
            method_args=[func, a],
            expected_args=[a],
            expected_kwargs={
                'function_urn': func.func_urn,
            }
        )


class TestFgVersion(TestFgProxy):
    def test_versions(self):
        func_urn = ('urn:fss:eu-de:45c274f200d2498683982c8741fb76ac:'
                    'function:default:access-mysql-js-1213-1737554083545:'
                    'latest')
        self.verify_list(
            self.proxy.versions,
            _version.Version,
            method_args=[func_urn],
            expected_args=[],
            expected_kwargs={
                'function_urn': func_urn.rpartition(":")[0]
            },
        )

    def test_publish_version(self):
        func = _function.Function(
            name='test',
            func_urn='urn:fss:eu-de:45c274f200d2498683982c8741fb76ac:'
                     'function:default:access-mysql-js-1213-1737554083545:'
                     'latest'
        )
        self.verify_create(
            self.proxy.publish_version,
            _version.Version,
            method_kwargs={
                'function': func,
                'name': 'test_event'
            },
            expected_kwargs={
                'function_urn': func.func_urn.rpartition(":")[0],
                'name': 'test_event'}
        )


class TestFgMetric(TestFgProxy):
    def test_metrics(self):
        self.verify_list(
            self.proxy.metrics,
            _metric.Metric,
        )

    def test_function_metrics(self):
        func = _function.Function(
            name='test',
            func_urn='urn:fss:eu-de:45c274f200d2498683982c8741fb76ac:'
                     'function:default:access-mysql-js-1213-1737554083545:'
                     'latest'
        )
        self.verify_list(
            self.proxy.function_metrics,
            _metric.Metric,
            method_args=[
                func,
                '1596686400000,1596686400000'
            ],
            expected_args=[],
        )


class TestFgLog(TestFgProxy):
    def test_get_lts_log_settings(self):
        func = _function.Function(
            name='test',
            func_urn='urn:fss:eu-de:45c274f200d2498683982c8741fb76ac:'
                     'function:default:access-mysql-js-1213-1737554083545:'
                     'latest'
        )
        self.verify_get(
            self.proxy.get_lts_log_settings,
            _log.Log,
            method_args=[
                func,
            ],
            expected_args=[],
            expected_kwargs={'function_urn': func.func_urn.rpartition(":")[0],
                             'requires_id': False}
        )

    def test_enable_lts_log(self):
        self.verify_create(
            self.proxy.enable_lts_log,
            _log.Log,
            method_kwargs={},
        )


class TestFgTemplate(TestFgProxy):
    def test_get_template(self):
        self.verify_get(
            self.proxy.get_template,
            _t.Template,
            method_args=["41d5d9ca-cea3-4ba9-b866-e30c46f45f1f"],
            expected_kwargs={
                "template_id": "41d5d9ca-cea3-4ba9-b866-e30c46f45f1f",
                "requires_id": False
            },
            expected_args=[]
        )


class TestFgReservedInstances(TestFgProxy):
    def test_update_event(self):
        func = _function.Function(
            name='test',
            func_urn='urn:fss:eu-de:45c274f200d2498683982c8741fb76ac:'
                     'function:default:access-mysql-js-1213-1737554083545:'
                     'latest'
        )
        self.verify_update(
            self.proxy.update_instances_number,
            _r.ReservedInstance,
            method_args=[func],
            expected_args=[],
            expected_kwargs={
                'function_urn': func.func_urn.rpartition(":")[0],
                'x': 1,
                'y': 2,
                'z': 3
            }
        )

    def test_reserved_instances_config(self):
        self.verify_list(
            self.proxy.reserved_instances_config,
            _r.ReservedInstance,
            method_args=[],
            expected_args=[],
        )

    def test_reserved_instances(self):
        self.verify_list(
            self.proxy.reserved_instances,
            _r.ReservedInstance,
            method_args=[],
            expected_args=[],
        )


class TestFgImportExport(TestFgProxy):
    def test_import_function(self):
        attrs = {
            'func_name': 'test',
            'file_type': 'zip',
            'file_name': 'test.zip',
            'file_code': 'UEsDBBQAAAAIAPBbPFpOs3AMAgEAAEwCAAAGAAAAZGVwLnB5jVHB'
                         'asMwDL37K4R3iSGMMtglsNNW2HH0B4oXq9SjtY2shJbSf5/tpl5y'
                         'GFQX23qS3nuWPQZPDD/ROyGEwR3stTMHpAZHdNxC7x3jiVUnIAXT'
                         '+XbJ8QRfmiIC7xGsCwND6YHGaNaqlhVom3PwVoieD16beCNQYj6O'
                         'bGrP4wL50Ro0kNtqRch4IzfYox0nsJPtjGExboM8kAMNhDF4F7Fi'
                         '90QSdKnJHDKy5iG+e4Oyg5fVql3C396cE7CTH9kOTUI6uPxJuMra'
                         'cp0RFinFvRmOITZ3CZNiPPUYGNblsD6pjoDzr/4sawEk8hRrvjy3'
                         'D9p5/d/OOs9JNiKnxasHLSzJlfgFUEsBAhQDFAAAAAgA8Fs8Wk6z'
                         'cAwCAQAATAIAAAYAAAAAAAAAAAAAAKSBAAAAAGRlcC5weVBLBQYA'
                         'AAAAAQABADQAAAAmAQAAAAA='
        }
        self.verify_create(
            self.proxy.import_function,
            _import.Import,
            method_kwargs={**attrs}
        )

    def test_export(self):
        function = _function.Function(id='test_function')
        self._verify(
            'otcextensions.sdk.function_graph.v2.export_function.'
            'Export._export',
            self.proxy.export_function,
            method_args=[function],
            expected_args=[self.proxy, function]
        )


class TestFgTrigger(TestFgProxy):

    def test_triggers(self):
        function = _function.Function(
            name='test',
            func_urn='urn:fss:eu-de:45c274f200d2498683982c8741fb76ac:'
                     'function:default:access-mysql-js-1213-1737554083545:'
                     'latest'
        )
        self.verify_list(
            self.proxy.triggers,
            _trigger.Trigger,
            method_kwargs={"function_urn": function.func_urn},
            expected_kwargs={
                "function_urn": function.func_urn.rpartition(":")[0]
            },
            expected_args=[]
        )

    def test_create_trigger(self):
        function = _function.Function(
            name='test',
            func_urn='urn:fss:eu-de:45c274f200d2498683982c8741fb76ac:'
                     'function:default:access-mysql-js-1213-1737554083545:'
                     'latest'
        )
        self.verify_create(self.proxy.create_trigger,
                           _trigger.Trigger,
                           method_args=[function],
                           method_kwargs={
                               "trigger_type_code": "TIMER",
                               "trigger_status": "ACTIVE",
                               "event_data": {
                                   "name": "Timer-l8v2",
                                   "schedule": "3m",
                                   "schedule_type": "Rate"
                               }
                           },
                           expected_kwargs={
                               "trigger_type_code": "TIMER",
                               "trigger_status": "ACTIVE",
                               "event_data": {
                                   "name": "Timer-l8v2",
                                   "schedule": "3m",
                                   "schedule_type": "Rate"
                               }
                           },
                           expected_args=[]
                           )

    def test_update_trigger(self):
        function = _function.Function(
            name='test',
            func_urn='urn:fss:eu-de:45c274f200d2498683982c8741fb76ac:'
                     'function:default:access-mysql-js-1213-1737554083545:'
                     'latest'
        )
        self._verify(
            'openstack.proxy.Proxy._update',
            self.proxy.update_trigger,
            method_args=[function.func_urn, 'TIMER', 'id'],
            method_kwargs={
                "trigger_status": "DISABLED",
            },
            expected_kwargs={
                "trigger_status": "DISABLED",
            },
            expected_args=[_trigger.Trigger]
        )

    def test_delete_trigger(self):
        function = _function.Function(
            func_urn='urn:fss:eu-de:45c274f200d2498683982c8741fb76ac:'
                     'function:default'
                     ':access-mysql-js-1213-1737554083545:latest')
        self._verify(
            'otcextensions.sdk.function_graph.v2.trigger.'
            'Trigger._delete_trigger',
            self.proxy.delete_trigger,
            method_args=[function, "TIMER", "id"],
            expected_args=[self.proxy, function, "TIMER", "id"],
        )

    def test_delete_triggers(self):
        function = _function.Function(
            func_urn='urn:fss:eu-de:45c274f200d2498683982c8741fb76ac:'
                     'function:default'
                     ':access-mysql-js-1213-1737554083545:latest')
        self._verify(
            'otcextensions.sdk.function_graph.v2.trigger.'
            'Trigger._delete_triggers',
            self.proxy.delete_all_triggers,
            method_args=[function.func_urn],
            expected_args=[self.proxy, function.func_urn.rpartition(":")[0]],
        )

    def test_get_trigger(self):
        self.verify_get(
            self.proxy.get_trigger,
            _trigger.Trigger,
            method_args=["urn", "code", "id"],
            expected_kwargs={
                "requires_id": False
            },
            expected_args=[]
        )


class TestFgAsyncNotifications(TestFgProxy):

    def test_async_notifications(self):
        function = _function.Function(
            name='test',
            func_urn='urn:fss:eu-de:45c274f200d2498683982c8741fb76ac:'
                     'function:default:access-mysql-js-1213-1737554083545:'
                     'latest'
        )
        self.verify_list(
            self.proxy.async_notifications,
            _async.Notification,
            method_kwargs={"function": function.func_urn},
            expected_kwargs={
                "function_urn": function.func_urn.rpartition(":")[0]
            },
            expected_args=[]
        )

    def test_configure_async_notification(self):
        function = _function.Function(
            name='test',
            func_urn='urn:fss:eu-de:45c274f200d2498683982c8741fb76ac:'
                     'function:default:access-mysql-js-1213-1737554083545:'
                     'latest'
        )
        self.verify_create(
            self.proxy.configure_async_notification,
            _async.Notification,
            method_args=[function],
            method_kwargs={
                "max_async_event_age_in_seconds": 1000,
            },
            expected_kwargs={
                "function_urn": function.func_urn.rpartition(":")[0],
                "max_async_event_age_in_seconds": 1000,
            },
            expected_args=[]
        )

    def test_delete_async_notification(self):
        function = _function.Function(
            func_urn='urn:fss:eu-de:45c274f200d2498683982c8741fb76ac:'
                     'function:default'
                     ':access-mysql-js-1213-1737554083545:latest')
        self.verify_delete(
            self.proxy.delete_async_notification,
            _async.Notification,
            method_args=[function],
            expected_args=[],
            expected_kwargs={
                'function_urn': function.func_urn.rpartition(":")[0]
            }
        )

    def test_all_versions_async_notifications(self):
        function = _function.Function(
            name='test',
            func_urn='urn:fss:eu-de:45c274f200d2498683982c8741fb76ac:'
                     'function:default:access-mysql-js-1213-1737554083545:'
                     'latest'
        )
        self.verify_list(
            self.proxy.all_versions_async_notifications,
            _async.Notification,
            method_kwargs={"function": function.func_urn},
            expected_kwargs={
                "function_urn": function.func_urn
            },
            expected_args=[]
        )

    def test_async_invocation_requests(self):
        function = _function.Function(
            name='test',
            func_urn='urn:fss:eu-de:45c274f200d2498683982c8741fb76ac:'
                     'function:default:access-mysql-js-1213-1737554083545:'
                     'latest'
        )
        self.verify_list(
            self.proxy.async_invocation_requests,
            _async.Requests,
            method_kwargs={"function": function.func_urn},
            expected_kwargs={
                "function_urn": function.func_urn.rpartition(":")[0]
            },
            expected_args=[]
        )

    def test_stop_async_invocation_request(self):
        function = _function.Function(
            func_urn='urn:fss:eu-de:45c274f200d2498683982c8741fb76ac:'
                     'function:default'
                     ':access-mysql-js-1213-1737554083545:latest')
        self._verify(
            'otcextensions.sdk.function_graph.v2.async_notification.'
            'Requests._stop',
            self.proxy.stop_async_invocation_request,
            method_args=[function],
            method_kwargs={"request_id": "123"},
            expected_args=[self.proxy],
            expected_kwargs={
                "function_urn": function.func_urn.rpartition(":")[0],
                "request_id": "123"
            }
        )



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
"""ModelArts service v1 action implementations"""
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.i18n import _

LOG = logging.getLogger(__name__)

INFER_TYPE_CHOICES = ["real-time", "batch"]


class ListServiceFlavors(command.Lister):
    _description = _("List Service Deployment Specifications.")
    columns = (
        "Specification",
        "Billing Spec",
        "Spec Status",
        "Is Open",
        "Is Free",
        "Over Quota",
        "Display EN",
        "Extend Params",
    )

    def get_parser(self, prog_name):
        parser = super(ListServiceFlavors, self).get_parser(prog_name)
        parser.add_argument(
            "--is-personal-cluster",
            action="store_true",
            help=_(
                "Whether to query the service deployment specifications "
                "supported by dedicated resource pool. "
                "The default value is false."
            ),
        )
        parser.add_argument(
            "--infer-type",
            metavar="{" + ",".join(INFER_TYPE_CHOICES) + "}",
            type=lambda s: s.lower(),
            choices=INFER_TYPE_CHOICES,
            help=_(
                "Inference mode. The default value is real-time. "
                "The value can be `real-time` or `batch`."
            ),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv1
        query_params = {}
        if parsed_args.is_personal_cluster:
            query_params["is_personal_cluster"] = True
        if parsed_args.infer_type:
            query_params["infer_type"] = parsed_args.infer_type

        data = client.service_flavors(**query_params)

        return (
            self.columns,
            (utils.get_item_properties(s, self.columns) for s in data),
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from openstack.tests.unit import base
from otcextensions.sdk.modelartsv1.v1 import service
from otcextensions.tests.unit.sdk.modelartsv1.v1 import examples
from otcextensions.tests.unit.sdk.utils import assert_attributes_equal

EXAMPLE = examples.SERVICE


EXAMPLE_CREATE = {
    "config": [
        {
            "model_id": "xxxmodel-idxxx",
            "weight": 70,
            "specification": "modelarts.vm.cpu.2u",
            "instance_count": 1,
            "envs": {"model_name": "mxnet-model-1", "load_epoch": "0"},
        },
        {
            "model_id": "xxxxxx",
            "weight": 30,
            "specification": "modelarts.vm.cpu.2u",
            "instance_count": 1,
        },
    ],
    "description": "mnist service",
    "infer_type": "real-time",
    "service_name": "mnist",
}


class TestService(base.TestCase):
    def setUp(self):
        super(TestService, self).setUp()

    def test_basic(self):
        sot = service.Service()

        self.assertEqual("/services", sot.base_path)
        self.assertEqual(None, sot.resource_key)
        self.assertEqual("services", sot.resources_key)

        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_commit)
        self.assertDictEqual(
            {
                "limit": "limit",
                "marker": "marker",
                "cluster_id": "cluster_id",
                "infer_type": "infer_type",
                "model_id": "model_id",
                "name": "service_name",
                "offset": "offset",
                "order": "order",
                "service_id": "service_id",
                "sort_by": "sort_by",
                "status": "status",
                "workspace_id": "workspace_id",
            },
            sot._query_mapping._mapping,
        )

    def test_make_it(self):
        sot = service.Service(**EXAMPLE)
        assert_attributes_equal(self, sot, EXAMPLE)

    def test_create_sot(self):
        sot = service.Service(**EXAMPLE_CREATE)
        assert_attributes_equal(self, sot, EXAMPLE_CREATE)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
import mock

from otcextensions.osclient.dns.v2 import ptr
from otcextensions.tests.unit.osclient.dns.v2 import fakes


class TestListPTR(fakes.TestDNS):

    objects = fakes.FakeFloatingIP.create_multiple(3)

    columns = (
        'id', 'ptrdname', 'address', 'status', 'description', 'ttl'
    )

    data = []

    for s in objects:
        data.append(fakes.gen_data(s, columns))

    def setUp(self):
        super(TestListPTR, self).setUp()

        self.cmd = ptr.ListPTR(self.app, None)

        self.client.ptrs = mock.Mock()
        self.client.api_mock = self.client.floating_ips

    def test_default(self):
        arglist = [
        ]

        verifylist = [
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [
            self.objects
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_once_with(
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))


class TestShowPTR(fakes.TestDNS):

    _data = fakes.FakeFloatingIP.create_one()

    columns = (
        'address', 'description', 'id',
        'ptrdname', 'ttl'
    )

    data = fakes.gen_data(_data, columns)

    def setUp(self):
        super(TestShowPTR, self).setUp()

        self.cmd = ptr.ShowPTR(self.app, None)

        self.client.get_ptr = mock.Mock()
        self.client.api_mock = self.client.get_floating_ip

    def test_default(self):
        arglist = [
            'fpid'
        ]

        verifylist = [
            ('floatingip_id', 'fpid')
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [
            self._data
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_once_with(
            'fpid'
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestSetPTR(fakes.TestDNS):

    _data = fakes.FakeFloatingIP.create_one()

    columns = (
        'address', 'description', 'id',
        'ptrdname', 'ttl'
    )

    data = fakes.gen_data(_data, columns)

    def setUp(self):
        super(TestSetPTR, self).setUp()

        self.cmd = ptr.SetPTR(self.app, None)

        self.client.create_ptr = mock.Mock()
        self.client.api_mock = self.client.set_floating_ip

    def test_default(self):
        arglist = [
            'fpid',
            'dname',
            '--description', 'descr',
            '--ttl', '350'
        ]

        verifylist = [
            ('floatingip_id', 'fpid'),
            ('ptrdname', 'dname'),
            ('description', 'descr'),
            ('ttl', 350)
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [
            self._data
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_once_with(
            description='descr',
            floating_ip='fpid',
            ptrdname='dname',
            ttl=350
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestDeletePTR(fakes.TestDNS):

    columns = (
        'address', 'description', 'floating_ip_id', 'id',
        'ptrdname', 'region', 'ttl'
    )

    def setUp(self):
        super(TestDeletePTR, self).setUp()

        self.cmd = ptr.DeletePTR(self.app, None)

        self.client.restore_ptr = mock.Mock()
        self.client.api_mock = self.client.unset_floating_ip

    def test_default(self):
        arglist = [
            'fpid'
        ]

        verifylist = [
            ('floatingip_id', 'fpid')
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [{}]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_once_with(
            'fpid'
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource

from otcextensions.sdk.auto_scaling.v1 import _base
from otcextensions.sdk import quotamixin


class Quota(quotamixin.QuotaProxyMixin, _base.Resource):
    """AutoScaling Quota resource"""
    resources_key = 'quotas.resources'
    base_path = '/quotas'

    # capabilities
    allow_list = True

    #: Properties
    #: Quota of type, current only ``alarm`` is valid
    type = resource.Body('type')
    #: Quota amount has been used
    used = resource.Body('used', type=int)
    #: Quota max amount
    max = resource.Body('max', type=int)
    #: Quota amount
    quota = resource.Body('quota', type=int)


class ScalingQuota(Quota):
    base_path = '/quotas/%(scaling_group_id)s'

    scaling_group_id = resource.URI('scaling_group_id')
